import platform

from PyQt5 import QtCore, QtWidgets

from ost import geom, gfx, gui, seq
from ost import settings
from ost import LogError, mol
from ost.bindings import WrappedTMAlign
from ost.bindings import msms
from ost.seq import alg
import ost
from ost.gui.scene.query_editor import QueryEditorWidget,QueryDialog

class SelectRefDialog(QtWidgets.QDialog):
  def __init__(self, ent_list, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.ent_list_ = ent_list
    vb = QtWidgets.QVBoxLayout()
    self.setLayout(vb)
    self.setWindowTitle("Select Reference Object")
    self.label = QtWidgets.QLabel("Please Select the Reference Object")
    self.list = QtWidgets.QTableWidget(self)
    self.list.horizontalHeader().setStretchLastSection(True)
    self.list.setColumnCount(2)
    self.list.verticalHeader().setVisible(False)
    self.list.horizontalHeader().setVisible(False)
    self.list.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
    self.list.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
    vb.addWidget(self.label)
    vb.addWidget(self.list)
    self.show_scores = QtWidgets.QCheckBox(self)
    self.show_scores.setText("Show Scores")
    self.show_scores.setChecked(True)
    vb.addWidget(self.show_scores)
    self.show_alignment = QtWidgets.QCheckBox(self)
    self.show_alignment.setText("Display Alignment")
    self.show_alignment.setChecked(False)
    vb.addWidget(self.show_alignment)
    hb = QtWidgets.QHBoxLayout()
    hb.setDirection(QtWidgets.QBoxLayout.LeftToRight)
    cancel_btn = QtWidgets.QPushButton("Cancel", self)
    load_btn = QtWidgets.QPushButton("Select", self)
    hb.addStretch(1)
    hb.addWidget(cancel_btn)
    hb.addWidget(load_btn)
    vb.addItem(hb)
    load_btn.setDefault(True)
    load_btn.clicked.connect(self.Select)
    cancel_btn.clicked.connect(self.reject)

    row = 0
    for ent in self.ent_list_:
      variant = QtCore.QVariant(ent)
      self.list.insertRow(row)
      new_item = QtWidgets.QTableWidgetItem("%i"%(row+1))
      new_item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
      new_item.setData(QtCore.Qt.UserRole,variant)
      self.list.setItem(row, 0, new_item)
      new_item = QtWidgets.QTableWidgetItem(ent.GetName())
      new_item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
      self.list.setItem(row, 1, new_item)
      row += 1  

    self.list.resizeColumnsToContents()
    
  def Select(self):
    items = self.list.selectedItems()
    for item in items:
      if(item.column()==0):
        ent = item.data(QtCore.Qt.UserRole)
        self.ent_list_.remove(ent)
        self.ent_list_.insert(0,ent)
    self.accept()
     
  def GetShowScores(self):
    return self.show_scores.isChecked()
  
  def GetDisplayAlignment(self):
    return self.show_alignment.isChecked()
  
  def GetEntities(self):
    return self.ent_list_

class ShowResultDialog(QtWidgets.QDialog):
  def __init__(self, ent_list, res_list, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.ent_list_ = ent_list
    vb = QtWidgets.QVBoxLayout()
    self.setLayout(vb)
    self.setWindowTitle("Alignment result")
    self.label = QtWidgets.QLabel("Alignment results with %s as reference"%ent_list[0].GetName())
    self.list = QtWidgets.QTableWidget(self)
    self.list.horizontalHeader().setStretchLastSection(True)
    self.list.setColumnCount(4)
    self.list.verticalHeader().setVisible(False)
    self.list.setHorizontalHeaderLabels (["Name","RMSD","TMScore",""])
    self.list.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
    vb.addWidget(self.label)
    vb.addWidget(self.list)
    hb = QtWidgets.QHBoxLayout()
    hb.setDirection(QtWidgets.QBoxLayout.LeftToRight)
    ok_btn = QtWidgets.QPushButton("OK", self)
    hb.addStretch(1)
    hb.addWidget(ok_btn)
    vb.addItem(hb)
    ok_btn.setDefault(True)
    ok_btn.clicked.connect(self.accept)

    for i in range(0, len(res_list)):
      self.list.insertRow(i)
      new_item = QtWidgets.QTableWidgetItem(ent_list[i+1].GetName())
      new_item.setFlags(QtCore.Qt.ItemIsEnabled)
      self.list.setItem(i, 0, new_item)
      new_item = QtWidgets.QTableWidgetItem("%.2f"%res_list[i].rmsd)
      new_item.setFlags(QtCore.Qt.ItemIsEnabled)
      self.list.setItem(i, 1, new_item)
      new_item = QtWidgets.QTableWidgetItem("%.2f"%res_list[i].tm_score)
      new_item.setFlags(QtCore.Qt.ItemIsEnabled)
      self.list.setItem(i, 2, new_item)
      new_item = QtWidgets.QTableWidgetItem()
      new_item.setFlags(QtCore.Qt.ItemIsEnabled)
      self.list.setItem(i, 3, new_item)
          
    self.list.resizeColumnsToContents()

class CalculateSurfaceSettingsDialog(QtWidgets.QDialog):
  def __init__(self, executable, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    vb = QtWidgets.QGridLayout()
    self.setLayout(vb)
    self.setWindowTitle("MSMS Surface Settings")
    msmsexe_label=QtWidgets.QLabel("executable")
    self.msmsexe_field=QtWidgets.QLineEdit()
    self.msmsexe_field.setText(executable)
    msmsexe_browsebutton=QtWidgets.QPushButton("Browse")
    vb.addWidget(msmsexe_label, 0, 0)
    vb.addWidget(self.msmsexe_field, 0, 1)
    vb.addWidget(msmsexe_browsebutton, 0, 2)
    surfname_label=QtWidgets.QLabel("surface name")
    self.surfname_field=QtWidgets.QLineEdit()
    self.surfname_field.setText("surface")
    vb.addWidget(surfname_label, 1, 0)
    vb.addWidget(self.surfname_field, 1, 1, 1, 2)
    density_label=QtWidgets.QLabel("density")
    self.density_spinbox=QtWidgets.QSpinBox()
    self.density_spinbox.setRange(1, 10)
    self.density_spinbox.setValue(4)
    vb.addWidget(density_label, 2, 0)
    vb.addWidget(self.density_spinbox, 2, 1, 1, 2)
    probe_label=QtWidgets.QLabel("probe radius")
    self.probe_spinbox=QtWidgets.QDoubleSpinBox()
    self.probe_spinbox.setDecimals(1)
    self.probe_spinbox.setSingleStep(0.1)
    self.probe_spinbox.setRange(0.3, 5.0)
    self.probe_spinbox.setValue(1.4)
    vb.addWidget(probe_label, 3, 0)
    vb.addWidget(self.probe_spinbox, 3, 1, 1, 2)
    selection_label=QtWidgets.QLabel("selection")
    self.selection_field=QtWidgets.QLineEdit()
    self.selection_field.setText("")
    vb.addWidget(selection_label, 4, 0)
    vb.addWidget(self.selection_field, 4, 1, 1, 2)
    self.noh_box=QtWidgets.QCheckBox("no hydrogens")
    vb.addWidget(self.noh_box, 5, 0)
    self.nohet_box=QtWidgets.QCheckBox("no hetatoms")
    vb.addWidget(self.nohet_box, 5, 1)
    self.nowat_box=QtWidgets.QCheckBox("no waters")
    vb.addWidget(self.nowat_box, 5, 2)
    
    cancel_btn = QtWidgets.QPushButton("Cancel", self)
    ok_btn = QtWidgets.QPushButton("OK", self)
    vb.addWidget(cancel_btn, 6, 1)
    vb.addWidget(ok_btn, 6, 2)
    
    msmsexe_browsebutton.clicked.connect(self.GetPath)
    ok_btn.clicked.connect(self.accept)
    cancel_btn.clicked.connect(self.reject)
    
  def GetPath(self):
    path, _ =QtWidgets.QFileDialog.getOpenFileName(self, "Choose MSMS Executable")
    if path!='':
      self.msmsexe_field.setText(path)

class SurfaceContextMenu(QtCore.QObject):
  def __init__(self, context_menu):
    try:
      settings_name="msms"
      self.executable=settings.Locate(settings_name)
    except settings.FileNotFound:
      self.executable=""
    QtCore.QObject.__init__(self, context_menu.qobject)
    self.action = QtWidgets.QAction("Calculate Surface", self)
    self.action.triggered.connect(self.CalculateSurface)
    context_menu.AddAction(self.action, gui.ContextActionType.ENTITY)
    
  def CalculateSurface(self):
    scene_selection = gui.SceneSelection.Instance()
    ent_list = list()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      ent_list.append(scene_selection.GetActiveNode(i))
    cssd = CalculateSurfaceSettingsDialog(self.executable)
    if(cssd.exec_()):
      self.__CalculateSurface(ent_list,
                              str(cssd.surfname_field.text()),
                              str(cssd.msmsexe_field.text()),
                              cssd.density_spinbox.value(),
                              cssd.probe_spinbox.value(),
                              str(cssd.selection_field.text()),
                              cssd.noh_box.isChecked(),
                              cssd.nohet_box.isChecked(),
                              cssd.nowat_box.isChecked())
      
  def __CalculateSurface(self,ent_list,name,msms_exe,density,
                         radius,selection,noh,nohet,nowat):
    for entity in ent_list:
      if isinstance(entity, gfx.Entity):
        try:
          s=msms.CalculateSurface(entity.view,
                                  msms_exe=msms_exe,
                                  density=density,
                                  radius=radius,
                                  selection=selection,
                                  no_hydrogens=noh,
                                  no_hetatoms=nohet,
                                  no_waters=nowat)[0]
          gfx.Scene().Add(gfx.Surface("%s_%s"%(entity.GetName(),name),s))
        except (RuntimeError, msms.MsmsProcessError):
          LogError("WARNING: Surface could not be calculated")
          return
        except UserWarning:
          LogError("WARNING: Entry with the same name already present in scene")
          return


class SecStructContextMenu(QtCore.QObject):
  def __init__(self, context_menu):
    QtCore.QObject.__init__(self, context_menu.qobject)
    self.action = QtWidgets.QAction("Assign Secondary Structure", self)
    self.action.triggered.connect(self.AssignSecondaryStructure)
    context_menu.AddAction(self.action, gui.ContextActionType.ENTITY)
    
  def AssignSecondaryStructure(self):
    scene_selection = gui.SceneSelection.Instance()
    ent_list = list()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      mol.alg.AssignSecStruct(node.view)
      node.UpdateView()

  
class AlignmentContextMenu(QtCore.QObject):

  def __init__(self, context_menu):
   
    QtCore.QObject.__init__(self, context_menu.qobject)
    self.action = QtWidgets.QAction("Align", self)
    self.action.triggered.connect(self.Align)
    context_menu.AddAction(self.action, gui.ContextActionType.ENTITY | gui.ContextActionType.MULTI)
    self.seq_viewer = None
    
  def Align(self):
    scene_selection = gui.SceneSelection.Instance()
    ent_list = list()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      ent_list.append(scene_selection.GetActiveNode(i))
    sd = SelectRefDialog(ent_list)
    if(sd.exec_()):
      self.__Align(sd.GetEntities(),sd.GetShowScores(), sd.GetDisplayAlignment())
        
  def __Align(self, ent_list, show_scores=True, display_alignment=False):
    node = ent_list[0]
    res_list = list()
    if isinstance(node, gfx.Entity):
      ref = node.view.chains[0]
      for i in range(1,len(ent_list)):
        node = ent_list[i]
        if isinstance(node, gfx.Entity):
          tm_result = WrappedTMAlign(node.view.chains[0], ref)
          res_list.append(tm_result)
          node.view.handle.EditXCS().ApplyTransform(tm_result.transform)
          node.UpdatePositions()
    if show_scores:
      self.__ShowScore(ent_list, res_list)
    if display_alignment:
      self.__DisplayAlignment(ent_list, res_list)
    
  def __ShowScore(self, ent_list, res_list):
    if(len(res_list)==1):
      res = res_list[0]
      string = "RMSD: %.2f, TMScore: %.2f"%(res.rmsd, res.tm_score)
      gui.GostyApp.Instance().perspective.StatusMessage(string)
    elif(len(res_list)>1):
      ShowResultDialog(ent_list, res_list).exec_()
      
  def __DisplayAlignment(self, ent_list, res_list):
    if(len(res_list)>0):
      ref_seq = seq.CreateSequence("%s (ref)"%ent_list[0].GetName(),
                                   res_list[0].alignment.GetSequence(1).GetGaplessString())
      aln_list = seq.AlignmentList()
      if(ref_seq.IsValid()):
        for i in range(0, len(res_list)):
          # WrappedTMAlign returns an alignment with second sequence
          # being reference... let's swap...
          new_aln = seq.CreateAlignment()
          new_aln.AddSequence(res_list[i].alignment.GetSequence(1))
          new_aln.AddSequence(res_list[i].alignment.GetSequence(0))
          new_aln.SetSequenceName(1, ent_list[i+1].GetName())
          aln_list.append(new_aln)
        alignment = alg.MergePairwiseAlignments(aln_list, ref_seq)
        gosty = gui.GostyApp.Instance()
        main_area = gosty.perspective.GetMainArea()
        if self.seq_viewer:
          self.seq_viewer.qobject.close()
        self.seq_viewer = gui.SequenceViewer(True)
        self.seq_viewer.AddAlignment(alignment)
        self.seq_viewer.ChangeDisplayMode("Highlight conservation 1")
        self.seq_viewer.Show()

class SelectMenuPoints(QtCore.QObject):
  def __init__(self, context_menu):
    QtCore.QObject.__init__(self, context_menu.qobject)
    action=QtWidgets.QAction("Select...", self)
    action.triggered.connect(self._Select)
    context_menu.AddAction(action, gui.ENTITY)
    action=QtWidgets.QAction("Copy Selection...", self)
    action.triggered.connect(self._CopyViews)
    context_menu.AddAction(action, gui.ENTITY)
    action=QtWidgets.QAction('Select...', self)
    action.triggered.connect(self._SelectViewsSameEntity)
    context_menu.AddAction(action, gui.ENTITY_VIEW|gui.VIEWS_SAME_OBJECT)    
  def _Select(self):
    scene_selection=gui.SceneSelection.Instance()
    ent=scene_selection.GetActiveNode(0)
    dialog=QueryDialog('Select...')
    if dialog.exec_():
      q=mol.Query(dialog.query)
      if q.IsValid():
        ent.selection=ent.view.Select(dialog.query, dialog.query_flags)
      else:
        ost.LogError("invalid query: %s" % q.error)

  def _UniqueName(self, ent):
    """
    Returns a name based on ent that is unique within the scene
    """
    ent_name=ent.GetName()
    num=2
    while True:
      candidate_name='%s-%d' % (ent_name, num)
      if not gfx.Scene().HasNode(candidate_name):
        return candidate_name
      num+=1

  def _SelectViewsSameEntity(self):

    union=gui.SceneSelection.Instance().GetViewUnion()
    dialog=QueryDialog('Select...')
    if dialog.exec_():
      q=mol.Query(dialog.query)    
      if q.IsValid():
        ve=gui.SceneSelection.Instance().GetViewEntity()
        ve.selection=union.Select(q, dialog.query_flags)
      else:    
        ost.LogError("invalid query: %s" % q.error)    
  
  def _CopyViews(self):
    views_to_add=[]
    scene_selection=gui.SceneSelection.Instance()
    ent=scene_selection.GetActiveNode(0)
    dialog=QueryDialog('Select...')
    if dialog.exec_():
      q=mol.Query(dialog.query)
      if q.IsValid():
        for i in range(scene_selection.GetActiveNodeCount()):
          ent=scene_selection.GetActiveNode(i)
          selected=ent.view.Select(q, dialog.query_flags)
          gfx_ent=gfx.Entity(self._UniqueName(ent),selected)
          gfx.Scene().Add(gfx_ent)
      else:    
        ost.LogError("invalid query: %s" % q.error)

def _InitContextMenu(app):
  _InitContextMenu.cm=app.scene_win.GetContextMenu()
  _InitContextMenu.am=AlignmentContextMenu(_InitContextMenu.cm)
  _InitContextMenu.sec=SecStructContextMenu(_InitContextMenu.cm)
  _InitContextMenu.sc=SurfaceContextMenu(_InitContextMenu.cm)
  _InitContextMenu.sm=SelectMenuPoints(_InitContextMenu.cm)



import unittest, os, sys
import ost
from ost import io, mol, settings
# check if we can import: fails if numpy or scipy not available
try:
  from ost.mol.alg.qsscoring import *
except ImportError:
  print("Failed to import qsscoring. Happens when numpy or scipy missing. " \
        "Ignoring test_qsscoring.py tests.")
  sys.exit(0)

from ost.mol.alg import lDDTSettings

def _LoadFile(file_name):
  """Helper to avoid repeating input path over and over."""
  return io.LoadPDB(os.path.join('testfiles', file_name))


class TestQSscore(unittest.TestCase):

  # TESTS base

  def test_QSscoreEntity(self):
    # use smallest test structure...
    ent = _LoadFile('3ia3.1.pdb')
    ent.SetName("my_ent")
    # create clean one
    qs_ent = QSscoreEntity(ent)
    self.assertTrue(qs_ent.is_valid)
    # check naming
    qs_ent.SetName("my_qs_ent")
    self.assertEqual(qs_ent.original_name, "my_ent")
    self.assertEqual(qs_ent.GetName(), "my_qs_ent")
    # check cleanup
    self.assertEqual(sorted(ch.name for ch in qs_ent.ent.chains), ['A', 'B'])
    self.assertEqual(qs_ent.removed_chains, ['_'])
    self.assertFalse(qs_ent.calpha_only)
    # check CA entity
    ca_ent = qs_ent.ca_entity
    self.assertEqual(sorted(ch.name for ch in ca_ent.chains), ['A', 'B'])
    self.assertEqual(ca_ent.residue_count, ca_ent.atom_count)
    self.assertEqual(ca_ent.Select('aname=CA').atom_count, ca_ent.atom_count)
    self.assertEqual(sorted(qs_ent.ca_chains.keys()), ['A', 'B'])
    for ch in ca_ent.chains:
      self.assertEqual(''.join([r.one_letter_code for r in ch.residues]),
                       str(qs_ent.ca_chains[ch.name]))
    # check chem groups
    self.assertEqual(sorted(qs_ent.chem_groups), [['A'], ['B']])
    # check contacts
    self.assertEqual(len(qs_ent.contacts['A']['B']), 45)
    self.assertAlmostEqual(qs_ent.contacts['A']['B'][23][127], 10.069, 2)
    self.assertEqual(len(qs_ent.contacts_ca['A']['B']), 42)
    self.assertAlmostEqual(qs_ent.contacts_ca['A']['B'][23][127], 10.471, 2)
    # check contact filtering
    old_contacts = qs_ent.contacts.copy()
    old_contacts_ca = qs_ent.contacts_ca.copy()
    qs_ent.contacts = old_contacts
    self.assertEqual(qs_ent.contacts, old_contacts)
    qs_ent.contacts_ca = old_contacts_ca
    self.assertEqual(qs_ent.contacts_ca, old_contacts_ca)
    dummy_contacts = {'A': {'B': {1: {2: 3.0, 4: 5.0}},
                            'C': {10: {20: 30.0, 40: 50.0}}},
                      'B': {'C': {100: {200: 300.0, 400: 500.0}}}}
    qs_ent.contacts = dummy_contacts
    self.assertEqual(qs_ent.contacts, {'A': {'B': {1: {2: 3.0, 4: 5.0}}}})
    self.assertEqual(qs_ent.contacts_ca, old_contacts_ca)
    qs_ent.contacts = old_contacts
    qs_ent.contacts_ca = dummy_contacts
    self.assertEqual(qs_ent.contacts, old_contacts)
    self.assertEqual(qs_ent.contacts_ca, {'A': {'B': {1: {2: 3.0, 4: 5.0}}}})

    # check chain removal for non-amino acid chains
    ent_extra = ent.Copy()
    edi = ent_extra.EditXCS()
    # classic ligand chain
    ch = edi.InsertChain('C')
    for _ in range(30):
      r = edi.AppendResidue(ch, 'HOH')
      edi.InsertAtom(r, 'O', ost.geom.Vec3())
    # DNA chain
    ch = edi.InsertChain('D')
    for _ in range(30):
      r = edi.AppendResidue(ch, 'A')
      edi.InsertAtom(r, 'P', ost.geom.Vec3())
    edi.UpdateICS()
    # ensure both removed
    qs_ent_test = QSscoreEntity(ent_extra)
    self.assertEqual(sorted(qs_ent_test.removed_chains), ['C', 'D', '_'])

    # invalid structures: monomers (or less) before or after cleaning
    ost.PushVerbosityLevel(-1)
    # empty view
    ent_empty = ent.CreateEmptyView()
    qs_ent_invalid = QSscoreEntity(ent_empty)
    self.assertFalse(qs_ent_invalid.is_valid)
    # monomer - should be valid
    ent_mono = ent.Select('cname=A')
    qs_ent_mono = QSscoreEntity(ent_mono)
    self.assertTrue(qs_ent_mono.is_valid)
    # short chain removed
    ent_short = ent.Select('cname=A or rnum<20')
    qs_ent_mono = QSscoreEntity(ent_short)
    self.assertTrue(qs_ent_mono.is_valid)
    self.assertEqual(sorted(qs_ent_mono.removed_chains), ['B', '_'])
    # non-AA chain removal
    ent_non_AA = ent_extra.Select('cname=C,D')
    qs_ent_invalid = QSscoreEntity(ent_non_AA)
    self.assertFalse(qs_ent_invalid.is_valid)
    self.assertEqual(sorted(qs_ent_invalid.removed_chains), ['C', 'D'])
    ost.PopVerbosityLevel()

    # exception when scoring with invalid QSscoreEntity
    with self.assertRaises(QSscoreError):
      qs_scorer_tst = QSscorer(qs_ent_invalid, qs_ent)
      qs_scorer_tst.global_score
    with self.assertRaises(QSscoreError):
      qs_scorer_tst = QSscorer(qs_ent, qs_ent_invalid)
      qs_scorer_tst.global_score


  # TESTS HETERO

  def test_HeteroCase1(self):
    # additional chains
    ent_1 = _LoadFile('4ux8.1.pdb') # A2 B2 C2, symmetry: C2
    ent_2 = _LoadFile('3fub.2.pdb') # A2 B2   , symmetry: C2
    qs_scorer = QSscorer(ent_1, ent_2)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    # check mappings
    self.assertEqual(qs_scorer.chem_mapping,
                     {('D', 'F'): ('B', 'D'), ('C', 'E'): ('A', 'C')})
    self.assertEqual(len(qs_scorer.symm_1), 2)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 2)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'C': 'A', 'E': 'C', 'D': 'B', 'F': 'D'})
    # check scoring
    self.assertAlmostEqual(qs_scorer.global_score, 0.825, 2)
    # without penalties the interface is the same
    self.assertAlmostEqual(qs_scorer.best_score, 1.0, 2)
    self._CheckScorer(qs_scorer)

  def test_HeteroCase1b(self):
    # as above but with assymetric unit of 3fub
    # -> no overlap found: check if extensive search can deal with it
    ent_1 = _LoadFile('4ux8.1.pdb')
    ent_2 = _LoadFile('3fub.au.pdb')
    qs_scorer = QSscorer(ent_1, ent_2)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    # check mappings
    self.assertEqual(qs_scorer.chem_mapping,
                     {('C', 'E'): ('A', 'C'), ('D', 'F'): ('D', 'B')})
    self.assertEqual(len(qs_scorer.symm_1), 2)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 2)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'C': 'A', 'E': 'C', 'D': 'B', 'F': 'D'})
    # check scoring
    self.assertAlmostEqual(qs_scorer.global_score, 0.356, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.419, 2)
    self._CheckScorer(qs_scorer)

    # enforce different chain mapping
    enforced_cm = {'C': 'C', 'E': 'A', 'D': 'D', 'F': 'B'}
    qs_scorer_2 = QSscorer(qs_scorer.qs_ent_1, qs_scorer.qs_ent_2)
    qs_scorer_2.chain_mapping = enforced_cm
    self.assertAlmostEqual(qs_scorer_2.global_score, 0.356, 2)
    self.assertAlmostEqual(qs_scorer_2.best_score, 0.419, 2)
    self.assertEqual(qs_scorer_2.chain_mapping, enforced_cm)
    self._CheckScorer(qs_scorer_2)

    # check if we get same mapping with restricted max_ca_per_chain_for_cm
    qs_scorer_2 = QSscorer(qs_scorer.qs_ent_1, qs_scorer.qs_ent_2)
    qs_scorer_2.max_ca_per_chain_for_cm = 50
    self.assertEqual(qs_scorer_2.chain_mapping, qs_scorer.chain_mapping)
    self.assertAlmostEqual(qs_scorer_2.global_score, qs_scorer.global_score, 2)
    self.assertAlmostEqual(qs_scorer_2.best_score, qs_scorer.best_score, 2)
    self._CheckScorer(qs_scorer_2)


  def test_HeteroCase2(self):
    # different stoichiometry
    ent_1 = _LoadFile('1efu.1.pdb') # A2 B2, symmetry: C2
    ent_2 = _LoadFile('4pc6.1.pdb') # A B  , no symmetry
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.3131, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.941, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
                     {('A', 'C'): ('A',), ('B', 'D'): ('B',)})
    self.assertEqual(len(qs_scorer.symm_1), 2)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 1)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'A', 'B': 'B'})

  def test_HeteroCase3(self):
    # more chains
    ent_1 = _LoadFile('2vjt.1.pdb') # A6 B6, symmetry: D3
    ent_2 = _LoadFile('3dbj.1.pdb') # A3 B3, symmetry: C3
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.359, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.958, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
                     {('A', 'C', 'E', 'G', 'I', 'K'): ('A', 'C', 'E'),
                      ('B', 'D', 'F', 'H', 'J', 'L'): ('B', 'D', 'F')})
    self.assertEqual(len(qs_scorer.symm_1), 6)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 3)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'A':'A', 'B':'B', 'G':'C', 'H':'D', 'I':'E', 'J':'F'})

    # user's symmetry groups
    symm_1 = [('E', 'D', 'C', 'B', 'A', 'F'), ('G', 'J', 'I', 'L', 'K', 'H')]
    symm_2 = [('A', 'B', 'C', 'D', 'E', 'F')]
    # use QSscoreEntity to go faster
    qs_scorer_symm = QSscorer(qs_scorer.qs_ent_1, qs_scorer.qs_ent_2)
    qs_scorer_symm.SetSymmetries(symm_1, symm_2)
    self.assertAlmostEqual(qs_scorer_symm.global_score,
                           qs_scorer.global_score, 2)
    self.assertAlmostEqual(qs_scorer_symm.best_score, qs_scorer.best_score, 2)
    self._CheckScorer(qs_scorer_symm)
    # check properties
    self.assertFalse(qs_scorer_symm.calpha_only)
    self.assertEqual(qs_scorer_symm.chem_mapping, qs_scorer.chem_mapping)
    self.assertEqual(qs_scorer_symm.symm_1, symm_1)
    self.assertEqual(qs_scorer_symm.symm_2, symm_2)
    self.assertEqual(qs_scorer_symm.chain_mapping,
                     {'C':'A', 'D':'B', 'E':'E', 'F':'F', 'K':'C', 'L':'D'})
    self._CheckScorer(qs_scorer_symm)

  def test_HeteroCase4(self):
    # inverted chains
    ent_1 = _LoadFile('3ia3.1.pdb') # AB, no symmetry
    ent_2 = _LoadFile('3ia3.2.pdb') # BA, no symmetry
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.980, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.980, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    # check mappings
    self.assertEqual(qs_scorer.chem_mapping, {('A',): ('B',), ('B',): ('A',)})
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'B', 'B': 'A'})

    # check superposition
    sup = qs_scorer.superposition
    self.assertAlmostEqual(sup.rmsd, 0.3372, 2)
    self.assertEqual(sup.view1.atom_count, 225)
    self.assertEqual(sup.view2.atom_count, 225)
    rmsd = mol.alg.CalculateRMSD(sup.view1, sup.view2, sup.transformation)
    self.assertAlmostEqual(sup.rmsd, rmsd, 2)

    # check if CA-only scoring is close to this
    ent_2_ca = ent_2.Select('aname=CA')
    # use QSscoreEntity to go faster
    qs_scorer_ca = QSscorer(qs_scorer.qs_ent_1, ent_2_ca)
    self.assertTrue(qs_scorer_ca.calpha_only)
    self.assertAlmostEqual(qs_scorer_ca.global_score, qs_scorer.global_score, 2)
    self.assertAlmostEqual(qs_scorer_ca.best_score, qs_scorer.best_score, 2)
    self._CheckScorer(qs_scorer_ca)
    # throw exception for messed up chains without CA atoms
    ent_2_no_ca = ent_2.Select('aname!=CA')
    with self.assertRaises(QSscoreError):
      qs_scorer_tst = QSscorer(qs_scorer.qs_ent_1, ent_2_no_ca)
      qs_scorer_tst.global_score
    ent_2_almost_no_ca = ent_2.Select('aname!=CA or cname=A')
    with self.assertRaises(QSscoreError):
      qs_scorer_tst = QSscorer(qs_scorer.qs_ent_1, ent_2_almost_no_ca)
      qs_scorer_tst.global_score


  def test_HeteroModel(self):
    # uncomplete model missing 2 third of the contacts
    ent_1 = _LoadFile('1eud_ref.pdb')               # AB, no symmetry
    ent_2 = _LoadFile('1eud_mdl_partial-dimer.pdb') # BA, no symmetry
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.323, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.921, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping, {('A',): ('A',), ('B',): ('B',)})
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'A', 'B': 'B'})


  # TESTS HOMO

  def test_HomoCase1(self):
    # different stoichiometry SOD
    ent_1 = _LoadFile('4dvh.1.pdb') # A2, symmetry: C2
    ent_2 = _LoadFile('4br6.1.pdb') # A4, symmetry: D2
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.147, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.866, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping, {('A', 'B'): ('B', 'C', 'D', 'A')})
    self.assertEqual(len(qs_scorer.symm_1), 1)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 2)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'B', 'B': 'A'})

  def test_HomoCase2(self):
    # broken cyclic symmetry
    ent_1 = _LoadFile('4r7y.1.pdb')   # A6, symmetry: C6
    ent_2 = ent_1.Select('cname=A,B') # A2, no symmetry
    qs_scorer = QSscorer(ent_1, ent_2)
    # only one interface over 6
    self.assertAlmostEqual(qs_scorer.global_score, 1/6., 2)
    # without penalties the interface is the same
    self.assertAlmostEqual(qs_scorer.best_score, 1.0, 2)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
                     {('A', 'B', 'C', 'D', 'E', 'F'): ('A', 'B')})
    self.assertEqual(len(qs_scorer.symm_1), 1)
    self.assertEqual(len(qs_scorer.symm_1[0]), 6)
    self.assertEqual(len(qs_scorer.symm_2), 1)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'A', 'B': 'B'})
    self._CheckScorer(qs_scorer)

    # using user symmetry groups
    symm_1 = [('A', 'B'), ('C', 'D'), ('E', 'F')]
    symm_2 = [('A', 'B')]
    # use QSscoreEntity to go faster
    qs_scorer_symm = QSscorer(qs_scorer.qs_ent_1, qs_scorer.qs_ent_2)
    qs_scorer_symm.SetSymmetries(symm_1, symm_2)
    self.assertEqual(qs_scorer_symm.global_score, qs_scorer.global_score)
    self.assertEqual(qs_scorer_symm.best_score, qs_scorer.best_score)
    # check properties
    self.assertFalse(qs_scorer_symm.calpha_only)
    self.assertEqual(qs_scorer_symm.chem_mapping, qs_scorer.chem_mapping)
    self.assertEqual(qs_scorer_symm.symm_1, symm_1)
    self.assertEqual(qs_scorer_symm.symm_2, symm_2)
    self.assertEqual(qs_scorer_symm.chain_mapping, qs_scorer.chain_mapping)
    self._CheckScorer(qs_scorer_symm)

    # try to put invalid symmetries
    qs_scorer_symm = QSscorer(qs_scorer.qs_ent_1, qs_scorer.qs_ent_2)
    triv_1 = [tuple(ch.name for ch in qs_scorer_symm.ent_to_cm_1.chains)]
    triv_2 = [tuple(ch.name for ch in qs_scorer_symm.ent_to_cm_2.chains)]
    qs_scorer_symm.SetSymmetries([], [])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    qs_scorer_symm.SetSymmetries([('A', 'B', 'C'), ('D', 'E', 'F')],
                                 [('A', 'B')])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    qs_scorer_symm.SetSymmetries([('A', 'B'), ('C', 'D'), ('E', 'F')],
                                 [('A', 'C')])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    qs_scorer_symm.SetSymmetries([('A', 'B'), ('C', 'D')],
                                 [('A', 'B')])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    qs_scorer_symm.SetSymmetries([('A', 'B'), ('C', 'D'), ('A', 'B')],
                                 [('A', 'B')])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    qs_scorer_symm.SetSymmetries([('A', 'B'), ('C', 'D', 'E', 'F')],
                                 [('A', 'B')])
    self.assertEqual(qs_scorer_symm.symm_1, triv_1)
    self.assertEqual(qs_scorer_symm.symm_2, triv_2)
    # check cleaning of extra chains
    qs_scorer_symm.SetSymmetries([('A','B'), ('C','D'), ('E','F'), ('G','H')],
                                 [('A', 'B', 'C')])
    self.assertEqual(qs_scorer_symm.symm_1, [('A','B'), ('C','D'), ('E','F')])
    self.assertEqual(qs_scorer_symm.symm_2, [('A', 'B')])


  def test_HomoCase3(self):
    # Multiple equivalent mappings: {'B': 'A'} or {'B': 'B'}
    ent_1 = _LoadFile('cameo_6vlp_server101_2.pdb')   # Monomer (B)
    ent_2 = _LoadFile('cameo_6vlp_target.pdb')   # Dimer (AB)
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertTrue(
        qs_scorer.chain_mapping == {'B': 'A'} or \
        qs_scorer.chain_mapping == {'B': 'B'}
    )


  # TEST EXTRA SCORES
  
  def test_lDDT(self):
    # check for penalized and unpenalized oligo lDDT
    ref = _LoadFile('4br6.1.pdb').Select('cname=A,B')
    mdl = _LoadFile('4br6.1.pdb')
    lddt_settings = lDDTSettings()
    qs_scorer = QSscorer(ref, mdl)
    lddt_oligo_scorer = qs_scorer.GetOligoLDDTScorer(lddt_settings, False)
    self.assertAlmostEqual(qs_scorer.global_score, 0.171, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer.oligo_lddt, 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer.weighted_lddt, 1.00, 2)
    self.assertEqual(len(lddt_oligo_scorer.sc_lddt), 2)
    self.assertAlmostEqual(lddt_oligo_scorer.sc_lddt[0], 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer.sc_lddt[1], 1.00, 2)
    # with penalty we account for extra model chains
    lddt_oligo_scorer_pen = qs_scorer.GetOligoLDDTScorer(lddt_settings, True)
    self.assertAlmostEqual(lddt_oligo_scorer_pen.oligo_lddt, 0.5213, 2)
    self.assertAlmostEqual(lddt_oligo_scorer_pen.weighted_lddt, 0.50, 2)
    self.assertEqual(len(lddt_oligo_scorer_pen.sc_lddt), 2)
    self.assertAlmostEqual(lddt_oligo_scorer_pen.sc_lddt[0], 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer_pen.sc_lddt[1], 1.00, 2)
    # flip them (use QSscoreEntity to go faster)
    qs_scorer2 = QSscorer(qs_scorer.qs_ent_2,
                          qs_scorer.qs_ent_1,
                          res_num_alignment=True)
    lddt_oligo_scorer2 = qs_scorer2.GetOligoLDDTScorer(lddt_settings, False)
    self.assertAlmostEqual(qs_scorer2.global_score, 0.171, 2)
    self.assertAlmostEqual(qs_scorer2.best_score, 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer2.weighted_lddt, 1.00, 2)
    self.assertEqual(len(lddt_oligo_scorer2.sc_lddt), 2)
    self.assertAlmostEqual(lddt_oligo_scorer2.sc_lddt[0], 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer2.sc_lddt[1], 1.00, 2)
    # without penalty we don't see extra chains
    self.assertAlmostEqual(lddt_oligo_scorer2.oligo_lddt, 1.00, 2)
    # with penalty we account for extra reference chains
    lddt_oligo_scorer2_pen = qs_scorer2.GetOligoLDDTScorer(lddt_settings, True)
    self.assertAlmostEqual(lddt_oligo_scorer2_pen.oligo_lddt, 0.4496, 2)
    self.assertAlmostEqual(lddt_oligo_scorer2_pen.weighted_lddt, 0.50, 2)
    self.assertEqual(len(lddt_oligo_scorer2_pen.sc_lddt), 2)
    self.assertAlmostEqual(lddt_oligo_scorer2_pen.sc_lddt[0], 1.00, 2)
    self.assertAlmostEqual(lddt_oligo_scorer2_pen.sc_lddt[1], 1.00, 2)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping, {('B', 'A'): ('B', 'C', 'D', 'A')})
    self.assertEqual(sorted(qs_scorer.symm_1), [('B', 'A')])
    self.assertEqual(sorted(qs_scorer.symm_2), [('B', 'A'), ('C', 'D')])
    self.assertEqual(qs_scorer.chain_mapping, {'A': 'A', 'B': 'B'})


  # TEST BIG STUFF and FANCY SYMMETRIES

  def test_HeteroBig(self):
    # comparing rubisco with half of it
    ent_1 = _LoadFile('4f0h.1.pdb')      # A8 B8, symmetry: D4
    ent_2 = _LoadFile('4f0h.1_half.pdb') # A4 B4, symmetry: C4
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 1/4., 2)
    self.assertAlmostEqual(qs_scorer.best_score, 1.0, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('A', 'C', 'E', 'G', 'I', 'K', 'M', 'O'): ('A', 'C', 'E', 'G'),
             ('B', 'D', 'F', 'H', 'J', 'L', 'N', 'P'): ('B', 'D', 'F', 'H')})
    self.assertEqual(len(qs_scorer.symm_1), 8)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 4)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'A': 'A', 'C': 'C', 'B': 'B', 'E': 'E',
                      'D': 'D', 'G': 'G', 'F': 'F', 'H': 'H'})

  def test_Capsid(self):
    ent_1 = _LoadFile('4gh4.2.pdb') # A5 B5 C5 D5, symmetry: C5
    ent_2 = _LoadFile('1qqp.2.pdb') # A5 B5 C5 D5, symmetry: C5
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.921, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.941, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('D', 'H', 'L', 'P', 'T'): ('D', 'H', 'L', 'P', 'T'),
             ('A', 'E', 'I', 'M', 'Q'): ('A', 'E', 'I', 'M', 'Q'),
             ('C', 'G', 'K', 'O', 'S'): ('C', 'G', 'K', 'O', 'S'),
             ('B', 'F', 'J', 'N', 'R'): ('B', 'F', 'J', 'N', 'R')})
    self.assertEqual(len(qs_scorer.symm_1), 5)
    self.assertEqual(len(qs_scorer.symm_1[0]), 4)
    self.assertEqual(len(qs_scorer.symm_2), 5)
    self.assertEqual(len(qs_scorer.symm_2[0]), 4)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'A': 'A', 'C': 'C', 'B': 'B', 'E': 'E', 'D': 'D',
                      'G': 'G', 'F': 'F', 'I': 'I', 'H': 'H', 'K': 'K',
                      'J': 'J', 'M': 'M', 'L': 'L', 'O': 'O', 'N': 'N',
                      'Q': 'Q', 'P': 'P', 'S': 'S', 'R': 'R', 'T': 'T'})

  def test_TetrahedralSymmetry(self):
    ent_1 = _LoadFile('1mog.1.pdb') # A12, symmetry: T
    ent_2 = _LoadFile('2cc6.1.pdb') # A12, symmetry: T
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.954, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.994, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L'): \
               ('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L')})
    self.assertEqual(len(qs_scorer.symm_1), 6)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 6)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    # chain mapping is ambiguous here so we cannot check it

  def test_Urease(self):
    ent_1 = _LoadFile('1e9y.1.pdb') # A12 B12, symmetry: T
    ent_2 = _LoadFile('1e9z.1.pdb') # A12 B12, symmetry: T
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.958, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.958, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('B', 'D', 'F', 'H', 'J', 'L', 'N', 'P', 'R', 'T', 'V', 'X'): \
               ('B', 'D', 'F', 'H', 'J', 'L', 'N', 'P', 'R', 'T', 'V', 'X'),
             ('A', 'C', 'E', 'G', 'I', 'K', 'M', 'O', 'Q', 'S', 'U', 'W'): \
               ('A', 'C', 'E', 'G', 'I', 'K', 'M', 'O', 'Q', 'S', 'U', 'W')})
    self.assertEqual(len(qs_scorer.symm_1), 12)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 12)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'A': 'A', 'C': 'W', 'B': 'B', 'E': 'E', 'D': 'X',
                      'G': 'G', 'F': 'F', 'I': 'I', 'H': 'H', 'K': 'K',
                      'J': 'J', 'M': 'M', 'L': 'L', 'O': 'O', 'N': 'N',
                      'Q': 'Q', 'P': 'P', 'S': 'S', 'R': 'R', 'U': 'C',
                      'T': 'T', 'W': 'U', 'V': 'D', 'X': 'V'})

  def test_C6SymmetryHetero(self):
    ent_1 = _LoadFile('3j3r.1.pdb') # A6 B6, symmetry: C6
    ent_2 = _LoadFile('3j3s.1.pdb') # A6 B6, symmetry: C6
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.559, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.559, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('A', 'B', 'C', 'D', 'E', 'F'): ('A', 'B', 'C', 'D', 'E', 'F'),
             ('G', 'H', 'I', 'J', 'K', 'L'): ('G', 'H', 'I', 'J', 'K', 'L')})
    self.assertEqual(len(qs_scorer.symm_1), 6)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 6)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)
    self.assertEqual(qs_scorer.chain_mapping,
                     {'A': 'A', 'C': 'C', 'B': 'B', 'E': 'E',
                      'D': 'D', 'G': 'G', 'F': 'F', 'I': 'I',
                      'H': 'H', 'K': 'K', 'J': 'J', 'L': 'L'})

  def test_OctahedralSymmetry(self):
    ent_1 = _LoadFile('3vcd.1.pdb') # A24, symmetry: O
    ent_2 = _LoadFile('4ddf.1.pdb') # A24, symmetry: O
    qs_scorer = QSscorer(ent_1, ent_2)
    self.assertAlmostEqual(qs_scorer.global_score, 0.975, 2)
    self.assertAlmostEqual(qs_scorer.best_score, 0.975, 2)
    self._CheckScorer(qs_scorer)
    # check properties
    self.assertFalse(qs_scorer.calpha_only)
    self.assertEqual(qs_scorer.chem_mapping,
            {('A', 'E', 'I', 'M', 'Q', 'U', 'B', 'C', 'D', 'F', 'G', 'H',
              'J', 'K', 'L', 'N', 'O', 'P', 'R', 'S', 'T', 'V', 'W', 'X'): \
               ('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
                'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X')})
    self.assertEqual(len(qs_scorer.symm_1), 12)
    self.assertEqual(len(qs_scorer.symm_1[0]), 2)
    self.assertEqual(len(qs_scorer.symm_2), 12)
    self.assertEqual(len(qs_scorer.symm_2[0]), 2)

    # Don't check for exact mapping, as structures are highly symmetric.
    # Several mappings are thus valid. Only check whether we found an
    # automated mapping with strict (stuff must really match)
    # chain_mapping_scheme
    self.assertEqual(qs_scorer.chain_mapping_scheme, 'strict')
    #self.assertEqual(qs_scorer.chain_mapping,
    #                 {'A': 'J', 'C': 'L', 'B': 'K', 'E': 'P', 'D': 'R',
    #                  'G': 'O', 'F': 'Q', 'I': 'T', 'H': 'X', 'K': 'S',
    #                  'J': 'U', 'M': 'G', 'L': 'I', 'O': 'N', 'N': 'H',
    #                  'Q': 'E', 'P': 'V', 'S': 'D', 'R': 'F', 'U': 'B',
    #                  'T': 'A', 'W': 'M', 'V': 'C', 'X': 'W'})


  ###########################################################################
  # HELPERS
  ###########################################################################
  
  def _CheckEntity(self, qs_ent, check_ca_contacts):
    # check internal consistency of QS entity
    self.assertTrue(qs_ent.is_valid)
    # CA chains
    ca_ent = qs_ent.ca_entity
    self.assertEqual(ca_ent.residue_count, ca_ent.atom_count)
    ca_names = sorted([ch.name for ch in ca_ent.chains])
    self.assertEqual(sorted(qs_ent.ca_chains.keys()), ca_names)
    # chem groups
    self.assertEqual(sorted([c for cg in qs_ent.chem_groups for c in cg]),
                     ca_names)
    # check contacts (only chain names)
    if check_ca_contacts:
      contacts = qs_ent.contacts_ca
    else:
      contacts = qs_ent.contacts
    for c1 in contacts:
      self.assertTrue(c1 in ca_names)
      for c2 in contacts[c1]:
        self.assertTrue(c2 in ca_names)
        self.assertLess(c1, c2)

  def _CheckScorer(self, qs_scorer):
    # check if we live up to our promises (assume: we did global score)
    qs_ent_1 = qs_scorer.qs_ent_1
    qs_ent_2 = qs_scorer.qs_ent_2
    # check QS entities
    self._CheckEntity(qs_ent_1, qs_scorer.calpha_only)
    self._CheckEntity(qs_ent_2, qs_scorer.calpha_only)
    self.assertNotEqual(qs_ent_1.GetName(), qs_ent_2.GetName())
    # check scorer attributes
    self.assertEqual(qs_scorer.calpha_only,
                     qs_ent_1.calpha_only or qs_ent_2.calpha_only)
    # check chem_mapping
    cn1 = set([c for cg in qs_ent_1.chem_groups for c in cg])
    cn2 = set([c for cg in qs_ent_2.chem_groups for c in cg])
    cm_names_1 = list()
    cm_names_2 = list()
    for cg1, cg2 in qs_scorer.chem_mapping.items():
      ch_ref = qs_scorer.ent_to_cm_1.FindChain(cg1[0])
      self.assertEqual(ch_ref.residue_count, ch_ref.atom_count)
      self.assertGreaterEqual(ch_ref.residue_count, 5)
      self.assertLessEqual(ch_ref.residue_count,
                           qs_scorer.max_ca_per_chain_for_cm)
      for ch_name in cg1:
        self.assertTrue(ch_name in cn1)
        ch = qs_scorer.ent_to_cm_1.FindChain(ch_name)
        self.assertTrue(ch.IsValid())
        self.assertEqual(ch_ref.residue_count, ch.residue_count)
        self.assertEqual(ch_ref.atom_count, ch.atom_count)
        cm_names_1.append(ch_name)
      for ch_name in cg2:
        self.assertTrue(ch_name in cn2)
        ch = qs_scorer.ent_to_cm_2.FindChain(ch_name)
        self.assertTrue(ch.IsValid())
        self.assertEqual(ch_ref.residue_count, ch.residue_count)
        self.assertEqual(ch_ref.atom_count, ch.atom_count)
        cm_names_2.append(ch_name)
    # check that there's no extra stuff in ent_to_cm_1/2
    cm_names_1.sort()
    self.assertEqual(sorted(ch.name for ch in qs_scorer.ent_to_cm_1.chains),
                     cm_names_1)
    cm_names_2.sort()
    self.assertEqual(sorted(ch.name for ch in qs_scorer.ent_to_cm_2.chains),
                     cm_names_2)
    # check symm_1 / symm_2
    # (>= 1 symm. group, all groups same length, all chains appear)
    self.assertGreaterEqual(len(qs_scorer.symm_1), 1)
    ref_symm_1 = qs_scorer.symm_1[0]
    self.assertTrue(all(len(cg) == len(ref_symm_1) for cg in qs_scorer.symm_1))
    self.assertEqual(sorted(c for cg in qs_scorer.symm_1 for c in cg),
                     cm_names_1)
    self.assertGreaterEqual(len(qs_scorer.symm_2), 1)
    ref_symm_2 = qs_scorer.symm_2[0]
    self.assertTrue(all(len(cg) == len(ref_symm_2) for cg in qs_scorer.symm_2))
    self.assertEqual(sorted(c for cg in qs_scorer.symm_2 for c in cg),
                     cm_names_2)
    # check chain_mapping
    # (all chains of ent with less chains mapped, each only once, chem_map)
    chm_names_1 = list(qs_scorer.chain_mapping.keys())
    chm_names_2 = list(qs_scorer.chain_mapping.values())
    self.assertEqual(len(chm_names_1), min(len(cm_names_1), len(cm_names_2)))
    self.assertEqual(len(set(chm_names_1)), len(chm_names_1))
    self.assertEqual(len(set(chm_names_2)), len(chm_names_2))
    for cg1, cg2 in qs_scorer.chem_mapping.items():
      for ch_name in cg1:
        if ch_name in qs_scorer.chain_mapping:
          self.assertTrue(qs_scorer.chain_mapping[ch_name] in cg2)
    # check alignments
    # (sorted and mapped and with views, all in mapped_residues)
    self.assertEqual(len(qs_scorer.alignments), len(chm_names_1))
    self.assertEqual(len(qs_scorer.alignments), len(qs_scorer.mapped_residues))
    for aln, ch_name in zip(qs_scorer.alignments, sorted(chm_names_1)):
      self.assertEqual(aln.sequences[0].name, ch_name)
      self.assertEqual(aln.sequences[1].name, qs_scorer.chain_mapping[ch_name])
      self.assertTrue(aln.sequences[0].HasAttachedView())
      self.assertTrue(aln.sequences[1].HasAttachedView())
      self.assertEqual(aln.sequences[0].attached_view.handle, qs_ent_1.ent)
      self.assertEqual(aln.sequences[1].attached_view.handle, qs_ent_2.ent)
      self.assertTrue(ch_name in qs_scorer.mapped_residues)
    # best_score / global_score in [0,1]
    self.assertGreaterEqual(qs_scorer.best_score, 0.0)
    self.assertLessEqual(qs_scorer.best_score, 1.0)
    self.assertGreaterEqual(qs_scorer.global_score, 0.0)
    self.assertLessEqual(qs_scorer.global_score, 1.0)


if __name__ == "__main__":
  try:
    settings.Locate(('clustalw', 'clustalw2'))
  except:
    print("Could not find ClustalW. Ignoring test_qsscoring.py tests.")
    sys.exit(0)
  from ost import testutils
  if testutils.SetDefaultCompoundLib():
    testutils.RunTests()
  else:
    print('No compound library available. Ignoring test_qsscoring.py tests.')



# code generation for subst_weight_matrix.cc
# loads BLOSUM substitution matrices from files provided by NCBI at:
# ftp://ftp.ncbi.nlm.nih.gov/blast/matrices/

def Generate(filename, matrix_name):

    with open(filename) as f:
        data = f.readlines()

    scores = dict()
    olcs = None    
    for line in data:
        if line.startswith('#'):
            continue
        if olcs is None:
            if " A " in line and " R " in line and " N " in line:
                # very high likelihood that this line contains the olcs
                olcs = line.strip().split()
            continue
        split_line = line.strip().split()
        if split_line[0] in olcs:
            olc = split_line[0]
            for score_idx, score in enumerate(split_line[1:]):
                scores[(olc, olcs[score_idx])] = score

    ost_olcs = "ABCDEFGHIKLMNPQRSTVWXYZ"
    print("short %s[23][23]={"%(matrix_name))
    for a in ost_olcs:
        score_string = "" 
        for b in ost_olcs:
            score = scores[(a,b)]
            if len(score_string) == 0:
                entry = [' ', ' ']
            else:
                entry = [',', ' ', ' ', ' ']
            for i in range(len(score)):
                entry[-1-i] = score[-1-i]
            score_string += ''.join(entry)
        print("  {%s},"%(score_string))
    print("};")
    print()

Generate("BLOSUM45", "RAW_BLOSUM45_DATA")
Generate("BLOSUM62", "RAW_BLOSUM62_DATA")
Generate("BLOSUM80", "RAW_BLOSUM80_DATA")
Generate("BLOSUM100", "RAW_BLOSUM100_DATA")





e=io.LoadPDB("../demos/data/sdh.pdb")
scene.Add(gfx.Entity("x",gfx.CPK,e.Select("chain=A")))

# fast spheres with 3D sprites, default
scene["x"].GetOptions(gfx.CPK).SetSphereMode(1)

# turn on realtime ambient occlusion
scene.SetAmbientOcclusion(True)

# strength of the AO effect, default is 1.0
scene.SetAmbientOcclusionWeight(1.0)

# AO algorithms 0-3, default is 1
scene.SetAmbientOcclusionMode(1)

# orientation
scene.SetRTC(geom.Mat4(0.827929,-0.460893,-0.319547,29.688,
                       0.174973,0.753608,-0.633609,19.9405,
                       0.532839,0.468671,0.704578,73.0075,
                       -5.4704,1.54823,-116,1))



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import mol
from datetime import datetime
from datetime import datetime
from PyQt5 import QtCore, QtWidgets, QtGui
from .color_select_widget import ColorSelectWidget
from .gradient_preset_widget import GradientPresetWidget
from .gradient_editor_widget import GradientPreview
from .gradient_editor_widget import GradientEdit
from .preset_editor_list_model import PresetEditorListModel
from .immutable_gradient_info_handler import ImmutableGradientInfoHandler
from .query_editor import QueryEditorWidget
from ost.mol import Prop
from ost.gfx import ByElementColorOp
from ost.gfx import ByChainColorOp
from ost.gfx import GradientLevelColorOp
from ost.gfx import UniformColorOp
from .preset import Preset
from .render_op import RenderOp
from .visibility_op import VisibilityOp

#Preset Editor
class PresetEditor(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
        
    self.setWindowTitle("Preset Editor")
    
    #Create Ui Elements
    self.list_view_ = QtWidgets.QListView()
    
    self.combo_box_ = QtWidgets.QComboBox()
    
    self.ufcow_=UniformColorOpWidget(self)
    self.glcow_=GradientLevelColorOpWidget(self)
    self.beow_=ByElementColorOpWidget(self)
    self.bcow_=ByChainColorOpWidget(self)
    self.row_=RenderOpWidget(self)
    self.vow_=VisibilityOpWidget(self)
    self.combo_box_.addItem("Uniform Color Operation", QtCore.QVariant(self.ufcow_))
    self.combo_box_.addItem("Gradient Operation", QtCore.QVariant(self.glcow_))
    self.combo_box_.addItem("By Element Operation", QtCore.QVariant(self.beow_))
    self.combo_box_.addItem("By Chain Operation", QtCore.QVariant(self.bcow_))
    self.combo_box_.addItem("RenderMode Operation", QtCore.QVariant(self.row_))
    self.combo_box_.addItem("Visibility Operation", QtCore.QVariant(self.vow_))
    
    self.add_button_ = QtWidgets.QPushButton("Add")
    
    #Create Model
    self.list_view_.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
    
    self.list_view_.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
    self.list_view_.customContextMenuRequested.connect(self.contextMenuEvent)
    
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.combo_box_,0,0,1,1)
    grid.addWidget(self.add_button_,0,1,1,1)
    grid.addWidget(self.list_view_,1,0,3,3)
    grid.addLayout(self.hbox_,5,0,1,3)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.add_button_.clicked.connect(self.Add)
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
  
    self.CreateContextMenu()
      
  def SetPreset(self, preset):
    self.list_model_ = PresetEditorListModel(preset, self)
    self.list_view_.setModel(self.list_model_)
  
  def CreateContextMenu(self):
    self.context_menu_ = QtWidgets.QMenu("Context menu", self)
    self.edit_ = QtWidgets.QAction("Edit", self.list_view_)
    self.remove_ = QtWidgets.QAction("Remove", self.list_view_)
    self.moveup_ = QtWidgets.QAction("Move Up", self.list_view_)
    self.movedown_ = QtWidgets.QAction("Move Down", self.list_view_)
    self.context_menu_.addAction(self.edit_)
    self.context_menu_.addAction(self.remove_)
    self.context_menu_.addAction(self.moveup_)
    self.context_menu_.addAction(self.movedown_)
    #Connect Signals with Slots
    self.edit_.triggered.connect(self.Edit)
    self.remove_.triggered.connect(self.Remove)
    self.moveup_.triggered.connect(self.MoveUp)
    self.movedown_.triggered.connect(self.MoveDown)

  def contextMenuEvent(self, pos):
    #ContextMenu
    index = self.list_view_.indexAt(pos)
    if index.isValid(): 
        self.context_menu_.popup(QtWidgets.QCursor.pos())

  def Add(self):
    dialog = self.combo_box_.itemData(self.combo_box_.currentIndex()).toPyObject()
    if(dialog.exec_()):
      row = self.list_model_.rowCount()
      op = dialog.GetOp()
      self.list_model_.AddItem(op, row)
  
  def Edit(self):
    current_index = self.list_view_.currentIndex()
    op = self.list_model_.GetOp(current_index)
    if isinstance(op, gfx.UniformColorOp):
       self.ufcow_.SetOp(op)
       if self.ufcow_.exec_():
         self.list_model_.SetItem(current_index, self.ufcow_.GetOp())
    elif isinstance(op, gfx.GradientLevelColorOp):
       self.glcow_.SetOp(op)
       if self.glcow_.exec_():
         self.list_model_.SetItem(current_index, self.glcow_.GetOp())
    elif isinstance(op, gfx.ByElementColorOp):
       self.beow_.SetOp(op)
       if self.beow_.exec_():
         self.list_model_.SetItem(current_index, self.beow_.GetOp())
    elif isinstance(op, gfx.ByChainColorOp):
       self.bcow_.SetOp(op)
       if self.bcow_.exec_():
         self.list_model_.SetItem(current_index, self.bcow_.GetOp())
    elif isinstance(op, RenderOp):
       self.row_.SetOp(op)
       if self.row_.exec_():
         self.list_model_.SetItem(current_index, self.row_.GetOp())
    elif isinstance(op, VisibilityOp):
       self.vow_.SetOp(op)
       if self.vow_.exec_():
         self.list_model_.SetItem(current_index, self.vow_.GetOp())
    
  def Remove(self):
    current_index = self.list_view_.currentIndex()
    self.list_model_.RemoveItem(current_index.row())
    
  def MoveDown(self):
    current_index = self.list_view_.currentIndex()
    if self.list_model_.GetLastRow != current_index.row():
      op = self.list_model_.GetOp(current_index)
      self.list_model_.RemoveItem(current_index.row())
      self.list_model_.AddItem(op, current_index.row()+1)
 
  def MoveUp(self):
    current_index = self.list_view_.currentIndex()
    if self.list_model_.GetLastRow != 0:
      op = self.list_model_.GetOp(current_index)
      self.list_model_.RemoveItem(current_index.row())
      self.list_model_.AddItem(op, current_index.row()-1)
           
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()

class UniformColorOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.query_editor_ = QueryEditorWidget(self)

    detail_label = QtWidgets.QLabel("Parts")
    self.detail_selection_cb_ = QtWidgets.QComboBox()
    self.detail_selection_cb_.addItem("Main and Detail",QtCore.QVariant(3))
    self.detail_selection_cb_.addItem("Main",QtCore.QVariant(2))
    self.detail_selection_cb_.addItem("Detail",QtCore.QVariant(1))
    
    color_label = QtWidgets.QLabel("Color")
    self.color_select_widget_ = ColorSelectWidget(30,30,QtGui.QColor("White"))    
    
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)

        
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(detail_label, 1, 0, 1, 1)
    grid.addWidget(self.detail_selection_cb_, 1, 1, 1, 1)
    grid.addWidget(color_label, 2, 0, 1, 1)
    grid.addWidget(self.color_select_widget_, 2, 1, 1, 1)
    grid.addLayout(self.hbox_,3,0,1,2)
    grid.setRowStretch(2, 1)
    self.setLayout(grid)
    
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    
  def GetOp(self):
    qv=mol.QueryViewWrapper(self.query_editor_.GetQuery(),self.query_editor_.GetQueryFlags())
    ufco = UniformColorOp(qv,gfx.RGBA(1,1,1,1))
    
    detail = self.detail_selection_cb_.itemData(self.detail_selection_cb_.currentIndex()).toPyObject()
    ufco.SetMask(detail)
    qcolor = self.color_select_widget_.GetColor()
    color=gfx.RGBAb(qcolor.red(),qcolor.green(),qcolor.blue(),qcolor.alpha())
    ufco.SetColor(color)
    return ufco

  def SetOp(self, ufco):
    self.query_editor_.SetQuery(ufco.GetSelection())
    self.query_editor_.SetQueryFlags(ufco.GetSelectionFlags())
    found=False
    for i in range(0,self.detail_selection_cb_.count()):
      mask = self.detail_selection_cb_.itemData(i).toPyObject()
      if mask == ufco.GetMask():
        self.detail_selection_cb_.setCurrentIndex(i)
        found = True
        break
    if not found:
      self.detail_selection_cb_.setCurrentIndex(0)
    
    color = ufco.GetColor()
    qcolor = QtGui.QColor(color.Red()*255, color.Green()*255, color.Blue()*255, color.Alpha()*255)
    self.color_select_widget_.SetColor(qcolor)

  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()

class GradientLevelColorOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    
    self.query_editor_ = QueryEditorWidget(self)
    
    detail_label = QtWidgets.QLabel("Parts")
    self.detail_selection_cb_ = QtWidgets.QComboBox()
    self.detail_selection_cb_.addItem("Main and Detail",QtCore.QVariant(3))
    self.detail_selection_cb_.addItem("Main",QtCore.QVariant(2))
    self.detail_selection_cb_.addItem("Detail",QtCore.QVariant(1))
    
    property_label = QtWidgets.QLabel("Property")
    self.property_edit_ = QtWidgets.QLineEdit()

    self.prop_combo_box_ = QtWidgets.QComboBox()
    self.prop_combo_box_.addItem("atom B-factor",QtCore.QVariant("abfac"))
    self.prop_combo_box_.addItem("average residue B-factor",QtCore.QVariant("rbfac"))
    self.prop_combo_box_.addItem("X-Coordinate",QtCore.QVariant("x"))
    self.prop_combo_box_.addItem("Y-Coordinate",QtCore.QVariant("y"))
    self.prop_combo_box_.addItem("Z-Coordinate",QtCore.QVariant("z"))
    self.prop_combo_box_.addItem("Residue Number",QtCore.QVariant("rnum"))
    self.prop_combo_box_.addItem("Atom Charge",QtCore.QVariant("acharge"))
    self.prop_combo_box_.addItem("Custom",QtCore.QVariant("custom"))
    

    level_label = QtWidgets.QLabel("Level")
    self.combo_box_ = QtWidgets.QComboBox(self);
    self.combo_box_.addItem("Atom",QtCore.QVariant(Prop.Level.ATOM))
    self.combo_box_.addItem("Residue",QtCore.QVariant(Prop.Level.RESIDUE))
    self.combo_box_.addItem("Chain",QtCore.QVariant(Prop.Level.CHAIN))
    self.combo_box_.addItem("Unspecified",QtCore.QVariant(Prop.Level.UNSPECIFIED))
        
    gradient_label = QtWidgets.QLabel("Gradient")
    self.gradient_preview_ = GradientPreview()    
    self.gradient_edit_ = GradientEdit(self.gradient_preview_,self)
    
    self.minmax_label_ = QtWidgets.QLabel("Min Max")
    self.auto_calc_ = QtWidgets.QCheckBox("Auto calculate")
    self.auto_calc_.setChecked(True)
    
    self.minv_label_ = QtWidgets.QLabel("Min Value")
    self.maxv_label_ = QtWidgets.QLabel("Max Value")
    
    self.minv_ = QtWidgets.QDoubleSpinBox(self)
    self.minv_.setDecimals(2)
    self.minv_.setMinimum(-9999.99)
    self.minv_.setValue(0)
    self.maxv_ = QtWidgets.QDoubleSpinBox(self)
    self.maxv_.setDecimals(2)
    self.maxv_.setValue(1)
    self.maxv_.setMinimum(-9999.99)
    
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)

        
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(detail_label, 1, 0, 1, 1)
    grid.addWidget(self.detail_selection_cb_, 1, 1, 1, 1)
    grid.addWidget(property_label, 2, 0, 1, 1)
    grid.addWidget(self.prop_combo_box_, 2, 1, 1, 1)
    grid.addWidget(self.property_edit_, 3, 1, 1, 1)
    grid.addWidget(level_label, 4, 0, 1, 1)
    grid.addWidget(self.combo_box_, 4, 1, 1, 1)
    grid.addWidget(gradient_label, 5, 0, 1, 1)
    grid.addWidget(self.gradient_preview_, 5, 1, 1, 1)
    grid.addWidget(self.gradient_edit_, 6, 1, 1, 1)
    grid.addWidget(self.gradient_edit_, 6, 1, 1, 1)
    grid.addWidget(self.minmax_label_,7,0,1,1)
    grid.addWidget(self.auto_calc_,7,1,1,1)
    grid.addWidget(self.minv_label_,8,0,1,1)
    grid.addWidget(self.minv_,8,1,1,1)
    grid.addWidget(self.maxv_label_,9,0,1,1)
    grid.addWidget(self.maxv_,9,1,1,1)
    grid.addLayout(self.hbox_,10,0,1,2)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.prop_combo_box_.currentIndexChanged.connect(self.UpdateGui)
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    self.auto_calc_.stateChanged.connect(self.UpdateGui)
    
    self.UpdateGui()
    
  def GetOp(self):
    gradient = self.gradient_edit_.GetGfxGradient()
    
    detail = self.detail_selection_cb_.itemData(self.detail_selection_cb_.currentIndex()).toPyObject()
    qv=mol.QueryViewWrapper(self.query_editor_.GetQuery(),self.query_editor_.GetQueryFlags())
    prop = str()
    level = Prop.Level()
    if(self.property_edit_.isEnabled()):
      prop = str(self.property_edit_.text())
      level = Prop.Level(self.combo_box_.itemData(self.combo_box_.currentIndex()).toPyObject())
    else:
      prop = str(self.prop_combo_box_.itemData(self.prop_combo_box_.currentIndex()).toPyObject())
    
    if(self.auto_calc_.isChecked()):
      return GradientLevelColorOp(qv, detail, prop, gradient, level)
    else:
      minv = self.minv_.value()
      maxv = self.maxv_.value()
      return GradientLevelColorOp(qv, detail, prop, gradient, minv, maxv, level)
      
    
  def SetOp(self, glco):
    self.query_editor_.SetQuery(glco.GetSelection())
    self.query_editor_.SetQueryFlags(glco.GetSelectionFlags())
    found=False
    for i in range(0,self.detail_selection_cb_.count()):
      mask = self.detail_selection_cb_.itemData(i).toPyObject()
      if mask == glco.GetMask():
        self.detail_selection_cb_.setCurrentIndex(i)
        found = True
        break
    if not found:
      self.detail_selection_cb_.setCurrentIndex(0)
    
    found = False
    for i in range(0,self.prop_combo_box_.count()):
      prop = str(self.prop_combo_box_.itemData(i).toPyObject())
      if prop == glco.GetProperty():
        self.prop_combo_box_.setCurrentIndex(i)
        found = True
    if not found:
      self.prop_combo_box_.setCurrentIndex(self.prop_combo_box_.count()-1)
      self.property_edit_.setText(glco.GetProperty())
      
    self.combo_box_.setCurrentIndex(glco.GetLevel())
    self.gradient_edit_.LoadGradient(ImmutableGradientInfoHandler.ConvertToQGradient(glco.GetGradient()))
    self.auto_calc_.setChecked(glco.GetCalculateMinMax());
    if(not glco.GetCalculateMinMax()):
      self.minv_.setValue(glco.GetMinV())
      self.maxv_.setValue(glco.GetMaxV())
    self.UpdateGui()
    
  def UpdateGui(self):
    prop = str(self.prop_combo_box_.itemData(self.prop_combo_box_.currentIndex()))
    if(prop == "custom"):
      self.combo_box_.setEnabled(True)
      self.property_edit_.setEnabled(True)
    else:
      self.combo_box_.setEnabled(False)
      self.property_edit_.setEnabled(False)
    
    if(self.auto_calc_.isChecked()):
      self.minv_.setEnabled(False)
      self.maxv_.setEnabled(False)
    else:
      self.minv_.setEnabled(True)
      self.maxv_.setEnabled(True)
      
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()
    
  def Update(self):
    pass #Do Nothing

class ByElementColorOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.query_editor_ = QueryEditorWidget(self)

    detail_label = QtWidgets.QLabel("Parts")
    self.detail_selection_cb_ = QtWidgets.QComboBox()
    self.detail_selection_cb_.addItem("Main and Detail",QtCore.QVariant(3))
    self.detail_selection_cb_.addItem("Main",QtCore.QVariant(2))
    self.detail_selection_cb_.addItem("Detail",QtCore.QVariant(1))
  
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(detail_label, 1, 0, 1, 1)
    grid.addWidget(self.detail_selection_cb_, 1, 1, 1, 1)
    grid.addLayout(self.hbox_,2,0,1,2)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    
  def GetOp(self):
    detail = self.detail_selection_cb_.itemData(self.detail_selection_cb_.currentIndex()).toPyObject()
    qv=mol.QueryViewWrapper(self.query_editor_.GetQuery(),self.query_editor_.GetQueryFlags())
    beco = ByElementColorOp(qv, detail)
    return beco

  def SetOp(self, beco):
    self.query_editor_.SetQuery(beco.GetSelection())
    self.query_editor_.SetQueryFlags(beco.GetSelectionFlags())
    found=False
    for i in range(0,self.detail_selection_cb_.count()):
      mask = self.detail_selection_cb_.itemData(i).toPyObject()
      if mask == beco.GetMask():
        self.detail_selection_cb_.setCurrentIndex(i)
        found = True
        break
    if not found:
      self.detail_selection_cb_.setCurrentIndex(0)
  
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()
    
    
class ByChainColorOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.query_editor_ = QueryEditorWidget(self)

    detail_label = QtWidgets.QLabel("Parts")
    self.detail_selection_cb_ = QtWidgets.QComboBox()
    self.detail_selection_cb_.addItem("Main and Detail",QtCore.QVariant(3))
    self.detail_selection_cb_.addItem("Main",QtCore.QVariant(2))
    self.detail_selection_cb_.addItem("Detail",QtCore.QVariant(1))
  
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(detail_label, 1, 0, 1, 1)
    grid.addWidget(self.detail_selection_cb_, 1, 1, 1, 1)
    grid.addLayout(self.hbox_,2,0,1,2)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    
  def GetOp(self):
    detail = self.detail_selection_cb_.itemData(self.detail_selection_cb_.currentIndex()).toPyObject()
    qv=mol.QueryViewWrapper(self.query_editor_.GetQuery(),self.query_editor_.GetQueryFlags())
    bcco = ByChainColorOp(qv, detail)
    return bcco

  def SetOp(self, bcco):
    self.query_editor_.SetQuery(bcco.GetSelection())
    self.query_editor_.SetQueryFlags(bcco.GetSelectionFlags())
    found=False
    for i in range(0,self.detail_selection_cb_.count()):
      mask = self.detail_selection_cb_.itemData(i).toPyObject()
      if mask == bcco.GetMask():
        self.detail_selection_cb_.setCurrentIndex(i)
        found = True
        break
    if not found:
      self.detail_selection_cb_.setCurrentIndex(0)
  
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()


class RenderOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.query_editor_ = QueryEditorWidget(self)
  
    self.keep_ = QtWidgets.QCheckBox("Keep")
    self.keep_.setChecked(False)
    
    render_label = QtWidgets.QLabel("Rendermode")
    self.render_modes_ = QtWidgets.QComboBox()
    self.render_modes_.addItem("Fast Bonds")
    self.render_modes_.addItem("Ball & Stick")
    self.render_modes_.addItem("Spheres")
    self.render_modes_.addItem("Fast Trace")
    self.render_modes_.addItem("Trace")
    self.render_modes_.addItem("Fast Spline")
    self.render_modes_.addItem("Smooth Tube")
    self.render_modes_.addItem("Helix & Strand Cartoon")
    
    self.render_modes_list_ = [gfx.RenderMode.SIMPLE,
                               gfx.RenderMode.CUSTOM,
                               gfx.RenderMode.CPK,
                               gfx.RenderMode.LINE_TRACE,
                               gfx.RenderMode.TRACE,
                               gfx.RenderMode.SLINE,
                               gfx.RenderMode.TUBE,
                               gfx.RenderMode.HSC]
    
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(self.keep_, 1, 1, 1, 1)
    grid.addWidget(render_label,2,0,1,1)
    grid.addWidget(self.render_modes_,2,1,1,1)
    grid.addLayout(self.hbox_,3,0,1,2)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    
  def GetOp(self):
    selection = self.query_editor_.GetQueryText()
    flags = self.query_editor_.GetQueryFlags()
    render_mode = self.render_modes_list_[self.render_modes_.currentIndex()]
    ro = RenderOp(render_mode, selection, flags, self.keep_.isChecked())
    return ro

  def SetOp(self, ro):
    self.query_editor_.SetQuery(ro.GetSelection())
    self.query_editor_.SetQueryFlags(ro.GetSelectionFlags())
    found=False
    for i in range(0,self.render_modes_.count()):
      render_mode = self.render_modes_list_[i]
      if render_mode == ro.GetRenderMode():
        self.render_modes_.setCurrentIndex(i)
        found = True
        break
    if not found:
      self.render_modes_.setCurrentIndex(0)
    self.keep_.setChecked(ro.IsKept())
    
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()
    
class VisibilityOpWidget(QtWidgets.QDialog):
  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.query_editor_ = QueryEditorWidget(self)
  
    self.visible_ = QtWidgets.QCheckBox("Visible")
    self.visible_.setChecked(True)
    
    self.hbox_ = QtWidgets.QHBoxLayout()
    self.ok_button_ = QtWidgets.QPushButton("OK")
    self.cancel_button_ = QtWidgets.QPushButton("Cancel")
    self.hbox_.addWidget(self.ok_button_)
    self.hbox_.addStretch()
    self.hbox_.addWidget(self.cancel_button_)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.query_editor_, 0, 0, 1, 2)
    grid.addWidget(self.visible_, 1, 1, 1, 1)
    grid.addLayout(self.hbox_,2,0,1,2)
    grid.setRowStretch(1, 1)
    self.setLayout(grid)
    
    self.ok_button_.clicked.connect(self.Ok)
    self.cancel_button_.clicked.connect(self.Cancel)
    
  def GetOp(self):
    selection = self.query_editor_.GetQueryText()
    flags = self.query_editor_.GetQueryFlags()
    vo = VisibilityOp(selection, flags, self.visible_.isChecked())
    return vo

  def SetOp(self, vo):
    self.query_editor_.SetQuery(vo.GetSelection())
    self.query_editor_.SetQueryFlags(vo.GetSelectionFlags())
    self.visible_.setChecked(vo.IsVisible())
  
  def Ok(self):
    self.accept()
    
  def Cancel(self):
    self.reject()



import unittest
from ost import *
from ost import settings
from ost.bindings import cadscore
from ost import testutils

class TestCADBindings(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")


  def testCADClassic(self):

    try:
      # all of the following need to be present
      cad_calc_path = settings.Locate('CADscore_calc.bash')  
      cad_read_g_path = settings.Locate('CADscore_read_global_scores.bash')  
      cad_read_l_path = settings.Locate('CADscore_read_local_scores.bash')
      executable_path = settings.Locate('voroprot2')
    except:
      print("Could not find CAD score classic executables: ignoring unit tests")
      return

    cad_result = cadscore.CADScore(self.protein, self.protein, 
                                   label="cad_classic")

    # model and reference are the same, we expect a global CAD score of 1
    self.assertEqual(cad_result.globalAA, 1.0)

    # one score per residue
    self.assertEqual(len(cad_result.localAA), len(self.protein.residues))

    # model and reference are the same, we expect local CAD scores of 0.0
    for score in cad_result.localAA.values():
      self.assertEqual(score, 0.0)

    # check whether this score is assigned to each residue as float property
    for r in self.protein.residues:
      self.assertTrue(r.HasProp("cad_classic"))
      self.assertEqual(r.GetFloatProp("cad_classic"), 0.0)


  def testCADVoronota(self):

    try:
      # all of the following need to be present
      voronota_cadscore_path = settings.Locate("voronota-cadscore")
      executable_path = settings.Locate("voronota")   
    except:
      print("Could not find CAD score voronota executables: ignoring unit tests")
      return

    cad_result = cadscore.CADScore(self.protein, self.protein, mode="voronota",
                                   label="cad_voronota")

    # model and reference are the same, we expect a global CAD score of 1
    self.assertEqual(cad_result.globalAA, 1.0)

    # one score per residue
    self.assertEqual(len(cad_result.localAA), len(self.protein.residues))

    # model and reference are the same, we expect local CAD scores of 1.0
    for score in cad_result.localAA.values():
      self.assertEqual(score, 1.0)

    # check whether this score is assigned to each residue as float property
    for r in self.protein.residues:
      self.assertTrue(r.HasProp("cad_voronota"))
      self.assertEqual(r.GetFloatProp("cad_voronota"), 1.0)
 

if __name__ == "__main__":
  testutils.RunTests()



import unittest
from ost import io, mol


class TestNonStandard(unittest.TestCase):
 
  def test_fastModified(self):
    # phoshoserine: test if we correctly strip off modifications
    tpl=io.LoadPDB('testfiles/sep.pdb')
    new_hdl=mol.CreateEntity();
    ed=new_hdl.EditXCS()
    c=ed.InsertChain('A')
    ed.AppendResidue(c, 'SER')

    err, has_cbeta=mol.alg.CopyConserved(tpl.residues[0], new_hdl.residues[0], ed)
    self.assertTrue(err)
    self.assertTrue(has_cbeta)
    residues=new_hdl.residues
    self.assertEqual(len(residues), 1)
    self.assertEqual(len(residues[0].atoms), 6)
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "N").IsValid())
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "CA").IsValid())
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "C").IsValid())
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "O").IsValid())
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "CB").IsValid())
    self.assertTrue(new_hdl.FindAtom("A", mol.ResNum(1), "OG").IsValid())
    
    
  def test_CBeta(self):
    # test if the dst residues contain cbeta, unless they are glycines
    tpl=io.LoadPDB('testfiles/cbeta.pdb')
    new_hdl=mol.CreateEntity();
    ed=new_hdl.EditXCS()
    c=ed.InsertChain('A')
    ed.AppendResidue(c, 'MET')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'HIS')
    err, has_cbeta=mol.alg.CopyConserved(tpl.residues[0], new_hdl.residues[0], ed)
    self.assertTrue(has_cbeta)
    self.assertTrue(err)
    err, has_cbeta=mol.alg.CopyConserved(tpl.residues[1], new_hdl.residues[1], ed)
    self.assertFalse(has_cbeta)
    self.assertTrue(err)
    err, has_cbeta=mol.alg.CopyConserved(tpl.residues[2], new_hdl.residues[2], ed)
    self.assertFalse(has_cbeta)
    self.assertTrue(err)
    err, has_cbeta=mol.alg.CopyConserved(tpl.residues[3], new_hdl.residues[3], ed)
    self.assertTrue(has_cbeta)
    self.assertTrue(err)
      
    residues=new_hdl.residues
    self.assertEqual(len(residues), 4)
    self.assertTrue(residues[0].FindAtom("CB").IsValid())
    self.assertFalse(residues[1].FindAtom("CB").IsValid())
    self.assertFalse(residues[2].FindAtom("CB").IsValid())
    self.assertTrue(residues[3].FindAtom("CB").IsValid())


  def test_CBetaFail(self):
    # make sure that we can handle cases where CB reconstruction fails

    # NOTES:
    # - SNN is (since June 2011) labeled as a modified ASN but has a weird
    #   backbone structure without any overlap in atom names with ASN
    #   -> we hence expect it to a) always fall back to CopyNonConserved
    #      and b) fail to copy any atoms (and hence also fail to)
    # - SNN also removes N of following residue which is expected to have an
    #   incomplete backbone which would make it impossible to create a CB pos.
    
    # source of file: residues A.198 and A.199 from PDB ID 2YHW
    tpl = io.LoadPDB('testfiles/cbeta_fail.pdb')
    new_hdl = mol.CreateEntity();
    ed = new_hdl.EditXCS()
    c = ed.InsertChain('A')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'ALA')
    ed.AppendResidue(c, 'ASN')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'ALA')
    # SNN to GLY
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[0], ed)
    self.assertFalse(err)
    self.assertEqual(new_hdl.residues[0].atom_count, 0)
    # SNN to ALA
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[1], ed)
    self.assertFalse(err)
    self.assertEqual(new_hdl.residues[1].atom_count, 0)
    # SNN to ASN
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[2], ed)
    self.assertFalse(err)
    self.assertEqual(new_hdl.residues[2].atom_count, 0)
    # GLY to GLY
    err = mol.alg.CopyResidue(tpl.residues[1], new_hdl.residues[3], ed)
    self.assertTrue(err)
    self.assertEqual(new_hdl.residues[3].atom_count, 3)
    self.assertFalse(new_hdl.residues[3].FindAtom("CB").IsValid())
    # GLY to ALA
    err = mol.alg.CopyResidue(tpl.residues[1], new_hdl.residues[4], ed)
    self.assertFalse(err)
    self.assertEqual(new_hdl.residues[4].atom_count, 3)
    self.assertFalse(new_hdl.residues[4].FindAtom("CB").IsValid())


  def test_CopyResidue(self):
    tpl = io.LoadPDB('testfiles/cbeta.pdb')
    new_hdl = mol.CreateEntity();
    ed = new_hdl.EditXCS()
    c = ed.InsertChain('A')
    ed.AppendResidue(c, 'MET')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'HIS')
    ed.AppendResidue(c, 'HIS')
    ed.AppendResidue(c, 'GLY')
    ed.AppendResidue(c, 'HIS')
    ed.AppendResidue(c, 'MET')
    
    # MET to MET
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[0], ed)
    self.assertTrue(err)
    #GLY to GLY
    err = mol.alg.CopyResidue(tpl.residues[1], new_hdl.residues[1], ed)
    self.assertTrue(err)
    # GLY to GLY
    err = mol.alg.CopyResidue(tpl.residues[2], new_hdl.residues[2], ed)
    self.assertTrue(err)
    #now we copy a HIS to a HIS
    err = mol.alg.CopyResidue(tpl.residues[3], new_hdl.residues[3], ed)
    self.assertTrue(err)
    # copy a GLY to a HIS
    err, has_cbeta = mol.alg.CopyNonConserved(tpl.residues[1], new_hdl.residues[4], ed)
    self.assertFalse(has_cbeta)
    # copy a MET to a GLY
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[5], ed)
    self.assertFalse(err)
    # copy a MET to a HIS
    err = mol.alg.CopyResidue(tpl.residues[0], new_hdl.residues[6], ed)
    self.assertFalse(err)
    # copy a GLY to a MET with adding CB
    err = mol.alg.CopyResidue(tpl.residues[1], new_hdl.residues[7], ed)
    self.assertFalse(err)
      
    residues = new_hdl.residues
    self.assertEqual(len(residues), 8)
    # MET to MET
    self.assertTrue(residues[0].FindAtom("CB").IsValid())
    #GLY to GLY
    self.assertFalse(residues[1].FindAtom("CB").IsValid())
    #now we copy a GLY to a GLY
    self.assertFalse(residues[2].FindAtom("CB").IsValid())
    #now we copy a HIS to a HIS
    self.assertTrue(residues[3].FindAtom("CB").IsValid())
    #now we copy a GLY to a HIS without adding CB
    self.assertFalse(residues[4].FindAtom("CB").IsValid())
    #now we copy a MET to a GLY
    self.assertFalse(residues[5].FindAtom("CB").IsValid())
    # copy a MET to a HIS
    self.assertTrue(residues[6].FindAtom("CB").IsValid())
    # copy a GLY to a MET with adding CB
    self.assertTrue(residues[7].FindAtom("CB").IsValid())
    

if __name__ == "__main__":
  from ost import testutils
  if testutils.SetDefaultCompoundLib():
    testutils.RunTests()
  else:
    print('No compound library available. Ignoring test_nonstandard.py tests.')



import math

e=io.LoadEntity('../entity/sdh.pdb')
c=e.FindChain('A')

edi=e.EditICS(mol.EditMode.BUFFERED_EDIT)
for r in c.GetResidueList():
  phi=r.GetPhiTorsion()
  psi=r.GetPsiTorsion()
  if phi.IsValid(): 
    edi.SetTorsionAngle(phi, math.radians(-60))    
  if psi.IsValid(): 
    edi.SetTorsionAngle(psi, math.radians(-45))    
  
edi.UpdateXCS()

g=gfx.Entity('helix', e.Select('aname=CA,N,O,C,CB'))
scene.Add(g)
scene.SetCenter(g.GetCenter())






import os
from ost.mol import mm

path_to_gromacs = "xyz"
charmm_ff = "charmm27.ff"
amber_ff = "amber03.ff"

charmm_path = os.path.join(path_to_gromacs, "share", "top", charmm_ff)
amber_path = os.path.join(path_to_gromacs, "share", "top", amber_ff)

# Let's first do CHARMM
reader = mm.FFReader(charmm_path)
reader.ReadGromacsForcefield()
reader.ReadResidueDatabase("aminoacids")
reader.ReadResidueDatabase("rna")
reader.ReadResidueDatabase("dna")
reader.ReadResidueDatabase("lipids")
reader.ReadITP("tip3p")
reader.ReadITP("ions")
ff = reader.GetForcefield()
ff.Save("CHARMM27.dat")

# Let's do amber
reader = mm.FFReader(amber_path)
reader.ReadGromacsForcefield()
reader.ReadResidueDatabase("aminoacids")
reader.ReadResidueDatabase("rna")
reader.ReadResidueDatabase("dna")
reader.ReadITP("tip3p")
reader.ReadITP("ions")
ff = reader.GetForcefield()
ff.Save("AMBER03.dat")




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import info
from .immutable_preset_info_handler import ImmutablePresetInfoHandler

#Preset Info Handler
class PresetInfoHandler(ImmutablePresetInfoHandler):
  def __init__(self, file_name):    
    ImmutablePresetInfoHandler.__init__(self, file_name)
  
  def StorePreset(self,preset):
    group = self.presets_.CreateGroup(ImmutablePresetInfoHandler.PRESET_GROUP_NAME)    
    preset.ToInfo(group)
    self.handle_.Export(self.file_name_)
    
  def RemovePreset(self, name):
    group_list = self.presets_.GetGroups(ImmutablePresetInfoHandler.PRESET_GROUP_NAME)
    group_to_del = None
    for group in group_list:
      if group.HasAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME):
        groupname = group.GetAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME)
        if name == groupname:
          group_to_del = group
          break
    
    if group_to_del != None:
      self.presets_.Remove(group_to_del)      
      self.handle_.Export(self.file_name_)
      
  def RenamePreset(self, old, new):
    group_list = self.presets_.GetGroups(ImmutablePresetInfoHandler.PRESET_GROUP_NAME)
    for group in group_list:
      if group.HasAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME):
        groupname = group.GetAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME)
        if old == groupname:
          group.SetAttribute(ImmutablePresetInfoHandler.NAME_ATTRIBUTE_NAME, new)
          self.handle_.Export(self.file_name_)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets
from .render_mode_widget import RenderModeWidget

#CPK Render Options
class CPKWidget(RenderModeWidget):
  def __init__(self, parent=None):
    RenderModeWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Spheres"
    
    #Defaults
    min_sphere_detail = 1
    max_sphere_detail = 20
    
    #Set Render Mode
    self.mode_ = gfx.RenderMode.CPK
        
    #Create Ui elements
    
    self.sphere_spinbox_ = QtWidgets.QSpinBox()
    self.sphere_spinbox_.setRange(min_sphere_detail, max_sphere_detail)
    cpk_mode_label = QtWidgets.QLabel("Rendering Mode")
    self.cpk_mode_ = QtWidgets.QComboBox()
    self.cpk_mode_.addItem("Triangles")
    self.cpk_mode_.addItem("3D Sprites")
    cpk_label = QtWidgets.QLabel(self.text_)
    font = cpk_label.font()
    font.setBold(True)
    
    sphere_label = QtWidgets.QLabel("Sphere Detail")
    grid = QtWidgets.QGridLayout()
    grid.addWidget(cpk_label,0,0,1,1)
    grid.addWidget(sphere_label, 1, 0, 1, 3)
    grid.addWidget(self.sphere_spinbox_, 1, 2, 1, 1)
    grid.setRowStretch(2,1)
    self.setLayout(grid)
  
    self.sphere_spinbox_.valueChanged.connect(self.UpdateSphereDetail)
    self.cpk_mode_.currentIndexChanged.connect(self.UpdateSphereMode)
    
    self.setMinimumSize(250,60)
    
  def UpdateSphereDetail(self, value):
    self.GetOptions().SetSphereDetail(value)
    self.ApplyOptions()

  def UpdateSphereMode(self, value):
    self.GetOptions().SetSphereMode(value)
    self.ApplyOptions()
    
  def UpdateGui(self,options):
    self.sphere_spinbox_.setValue(options.GetSphereDetail())
    self.cpk_mode_.setCurrentIndex(options.GetSphereMode())

  def GetText(self):
    return self.text_

  def GetRenderMode(self):
    return self.mode_



"""
This module defines a widget used for visualization of trajectories.

"""

__all__=('TrajWidget','SetTimeUnit','AddTrajectory','RemoveTrajectory','SetSpeed','ReleaseGfxEntity','FixGfxEntity')

#-------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#-------------------------------------------------------------------------------

import PyQt5 as _PyQt5
from PyQt5.QtCore import Qt as _Qt
from PyQt5.QtWidgets import QWidget as _QWidget
from PyQt5.QtWidgets import QLabel as _QLabel
from PyQt5.QtWidgets import QPushButton as _QPushButton
import ost as _ost
import math as _math


class TrajWidget(_QWidget):
  def __init__(self, trajlist=None, golist=None, parent=None,ref_index=0):
    _QWidget.__init__(self, parent, _Qt.Tool)
    self.setFocusPolicy(_Qt.ClickFocus)
    self.ref_index_=ref_index
    self.trajlist_=trajlist
    self.golist_=golist
    self.ehlist_=[go.GetView().handle for go in self.golist_]
    self.index_dict={}
    self.fix_dict={}
    self.modifiers=None
    for i,go in enumerate(self.golist_):
      self.index_dict[go.name]=i
    vb=_PyQt5.QtWidgets.QVBoxLayout()
    hb=_PyQt5.QtWidgets.QHBoxLayout()
    hb1=_PyQt5.QtWidgets.QHBoxLayout()
    hb2=_PyQt5.QtWidgets.QHBoxLayout()
    hb3=_PyQt5.QtWidgets.QHBoxLayout()
    hb4=_PyQt5.QtWidgets.QHBoxLayout()
    self.callback=None   
    self._slider=_PyQt5.QtWidgets.QSlider(self)
    self._slider.setOrientation(_Qt.Horizontal)
    self._speed_slider=_PyQt5.QtWidgets.QSlider(self)
    self._speed_slider.setOrientation(_Qt.Horizontal)
    self._speedLabel=_QLabel(self)
    self._speedLabel.setText('Speed:')
    self._speedLabel.setAlignment(_Qt.AlignLeft)
    self._play=_PyQt5.QtWidgets.QToolButton(self)
    self._repeat=_PyQt5.QtWidgets.QCheckBox(self)
    self._frame=_QLabel(self)
    self._frameNo=_QLabel(self)
    self._frameEnd=_QLabel(self)
    self._repeat.setText('Repeat')
    self._slider.setTracking(True)
    self._play.setText('Play')
    self._play.setCheckable(True)
    self._frame.setText('Frame: ')
    self._frameNo.setNum(0)
    self._frameNo.setAlignment(_Qt.AlignRight)
    self._frameEnd.setAlignment(_Qt.AlignLeft)
    self._timeLabel=_QLabel(self)
    self._timeNo=_QLabel(self)
    self._timeEnd=_QLabel(self)
    self._timeUnit=_QLabel(self)
    self._timeLabel.setText('Time: ')
    self._timeNo.setNum(0)
    self._timeNo.setAlignment(_Qt.AlignRight)
    self._timeEnd.setAlignment(_Qt.AlignLeft)
    self._timeUnit.setAlignment(_Qt.AlignLeft)
    self._speed_slider.setTracking(True)
    self._speed_slider_min=-50
    self._speed_slider_max=-10
    self._speed_slider.setRange(self._speed_slider_min,self._speed_slider_max)
    self._right_arrow=_QPushButton(">")
    self._right_arrow2=_QPushButton(">>")
    self._right_end=_QPushButton(">|")
    self._left_arrow=_QPushButton("<")
    self._left_arrow2=_QPushButton("<<")
    self._left_end=_QPushButton("|<")
    self._align_selection=_PyQt5.QtWidgets.QLineEdit()
    self._align=_QPushButton("Align")
    self._align_label=_QLabel(self)
    self._align_label.setText('Selection:')
    self._ref_entity_selection=_PyQt5.QtWidgets.QComboBox()
    for go in self.golist_:self._ref_entity_selection.addItem(go.name)
    self._ref_entity_selection_label=_QLabel(self)
    self._ref_entity_selection_label.setText('Ref. Entity:')
    hb.addWidget(self._play)
    hb.addWidget(self._repeat)
    hb.addWidget(self._ref_entity_selection_label)
    hb.addWidget(self._ref_entity_selection)
    hb1.addWidget(self._left_end)
    hb1.addWidget(self._left_arrow2)
    hb1.addWidget(self._left_arrow)
    hb1.addWidget(self._right_arrow)
    hb1.addWidget(self._right_arrow2)
    hb1.addWidget(self._right_end)
    hb2.addWidget(self._frame)
    hb2.addWidget(self._frameNo)
    hb2.addWidget(self._frameEnd)
    hb2.addWidget(self._timeLabel)
    hb2.addWidget(self._timeNo)
    hb2.addWidget(self._timeEnd)
    hb2.addWidget(self._timeUnit)
    hb3.addWidget(self._speedLabel)
    hb3.addWidget(self._speed_slider)
    hb4.addWidget(self._align_label)
    hb4.addWidget(self._align_selection)
    hb4.addWidget(self._align)
    self.setLayout(vb)
    vb.addLayout(hb)
    vb.addLayout(hb1)
    vb.addWidget(self._slider)
    vb.addLayout(hb2)
    vb.addLayout(hb3)
    vb.addLayout(hb4)
    self._SetSpeedSliderPos(self._speed_slider_min+0.5*(self._speed_slider_max-self._speed_slider_min))
    self._play.toggled.connect(self._TogglePlay)
    self._slider.valueChanged.connect(self._SliderValueChanged)
    self._speed_slider.valueChanged.connect(self._SpeedSliderValChanged)
    self._right_end.clicked.connect(self._RightEndClicked)
    self._right_arrow2.clicked.connect(self._Right2Clicked)
    self._right_arrow.clicked.connect(self._RightClicked)
    self._left_arrow2.clicked.connect(self._Left2Clicked)
    self._left_arrow.clicked.connect(self._LeftClicked)
    self._left_end.clicked.connect(self._LeftEndClicked)
    self._align.clicked.connect(self._AlignClicked)
    self._ref_entity_selection.currentIndexChanged.connect(self._EntitySelected)
    self._slider.setMinimum(0)
    self.frame_number_=self.traj_.GetFrameCount()-1
    self.timestep_=self.traj_.GetDelta()
    self.SetTimeUnit('ns')
    self.SetReferenceIndex(0)
    
  def _p2u(self,u):
    if u=="s":
      return 1.0e-12
    elif u=="ms":
      return 1.0e-9
    elif u=="us":
      return 1.0e-6
    elif u=="ns":
      return 1.0e-3
    elif u=="ps":
      return 1.0
    elif u=="fs":
      return 1.0e3
    raise RuntimeError("expected one of s,ms,us,ns,ps or fs for unit")

  def SetTimeUnit(self,u):
    self._time_prefactor=self._p2u(u)
    self._timeEnd.setText('/ '+ str((self.frame_number_-1)*self.timestep_*self._time_prefactor))
    self._timeNo.setNum(self.current_frame*self.timestep_*self._time_prefactor)
    self._timeUnit.setText('['+u+']')
  
  def _SetSpeedSliderPos(self,pos):
    self._speed_slider.setSliderPosition(pos)
    self._SpeedSliderValChanged(pos)
    
  def _SpeedSliderValChanged(self,speed_pos):
    self.time=_math.exp(-0.15*speed_pos)
    if self._play.isChecked():
      self._TogglePlay(False)
      self._TogglePlay(True)
  
  def _SetTime(self,t):
    self.time=t
    self._speed_slider.setSliderPosition(-1./0.15*_math.log(t))
    if self._play.isChecked():
      self._TogglePlay(False)
      self._TogglePlay(True)
  
  
  def _SliderValueChanged(self, pos):
    self.current_frame=pos
    for traj,go in zip(self.trajlist_,self.golist_):
      if go.name in self.fix_dict:continue
      traj.CopyFrame(self.current_frame)
      go.UpdatePositions()

  def _GetCurrentFrame(self):
    return self._slider.sliderPosition()
    
  def _SetCurrentFrame(self, pos):
    if self._slider.maximum()<pos:
      if self._repeat.isChecked():
        pos=0
      else:
        pos=self._slider.maximum()
    self._slider.setSliderPosition(pos)
    self._frameNo.setNum(pos)
    self._timeNo.setNum(pos*self.timestep_*self._time_prefactor)
    
  current_frame=property(_GetCurrentFrame, _SetCurrentFrame)
  
  def _GetReferenceTraj(self):
    return self.trajlist_[self.ref_index_]
  traj_=property(_GetReferenceTraj)
  
  def timerEvent(self, event):
    #if self.callback:
    #  self.callback(self.golist_)
    self.current_frame+=1
    for go in self.golist_:
      if go.name in self.fix_dict:continue
      go.BlurSnapshot()
      go.UpdatePositions()
  
  def _TogglePlay(self, playing):
    if playing:
      self.timer_id_=self.startTimer(self.time)
    else:
      self.killTimer(self.timer_id_)
  
  def _LeftClicked(self):
    if self.current_frame>0:
      self.current_frame-=1
  
  def _RightClicked(self):
    if self.current_frame<self.frame_number_-1:
      self.current_frame+=1
  
  def _Left2Clicked(self):
    if self.current_frame>=10:
      self.current_frame-=10
  
  def _Right2Clicked(self):
    if self.current_frame<self.frame_number_-10:
      self.current_frame+=10
  
  def _LeftEndClicked(self):
    if self.current_frame>0:
      self.current_frame=0
  
  def _RightEndClicked(self):
    if self.current_frame<self.frame_number_-1:
      self.current_frame=self.frame_number_-1
    
  def _AlignClicked(self):
    ref_eh=self.ehlist_[self.ref_index_]
    ref_v=ref_eh.Select(str(self._align_selection.text()))
    if ref_v.GetAtomCount()<=3:
      print('not enough atoms for alignment')
      return
    for i,t,eh in zip(list(range(len(self.trajlist_))),self.trajlist_,self.ehlist_):
      t=_ost.mol.alg.SuperposeFrames(t,eh.Select(str(self._align_selection.text())),ref_v)
      self.trajlist_[i]=t
      
  def _SetBlur(self, blur):
    for go in self.golist_:
      go.SetBlur(blur)

  def _GetBlur(self):
    return self.gfx_entity.GetBlur()

  blur=property(_GetBlur, _SetBlur)

  def SetReferenceIndex(self,ref_index):
    if type(ref_index)==type(''):
      self.ref_index_=self.index_dict[ref_index]
    else:
      self.ref_index_=ref_index
    self.frame_number_=self.traj_.GetFrameCount()
    self.timestep_=self.traj_.GetDelta()
    self._slider.setMaximum(self.frame_number_-1)
    self._frameEnd.setText('/ '+ str(self.frame_number_-1))
    self._timeEnd.setText('/ '+ str((self.frame_number_-1)*self.timestep_*self._time_prefactor))
    self._ref_entity_selection.setCurrentIndex(self.ref_index_)
  
  def _EntitySelected(self,index):
    ref_index=self.index_dict[str(self._ref_entity_selection.currentText())]
    self.SetReferenceIndex(ref_index)
  
  def keyPressEvent(self, event):
    key=event.key()
    if event.modifiers()==_Qt.ControlModifier:
      self.modifiers=event.modifiers()
    if self.modifiers==_Qt.ControlModifier:
      if key==_Qt.Key_Left:
        self._Left2Clicked()
      elif key==_Qt.Key_Right:
        self._Right2Clicked()
    else:
      if key==_Qt.Key_Left:
        self._LeftClicked()
      elif key==_Qt.Key_Right:
        self._RightClicked()
      elif key==_Qt.Key_Space:
        self._play.setChecked(not self._play.isChecked())
      else:
        _QWidget.keyPressEvent(self, event)
  
  def keyReleaseEvent(self, event):
    if event.key()==_Qt.Key_Control:
      self.modifiers=None
    else:
      _QWidget.keyPressEvent(self, event)
  
  def FixGfxEntity(self,index,frame_number=None):
    if not frame_number:frame_number=self.current_frame
    if type(index)==type(''):
      self.fix_dict[index]=frame_number
      i=self.index_dict[index]
    else:
      i=index
      self.fix_dict[self.golist_[i].name]=frame_number
    self.trajlist_[i].CopyFrame(frame_number)
    self.golist_[i].UpdatePositions()
  
  def ReleaseGfxEntity(self,index):
    if type(index)==type(''):self.fix_dict.pop(index)
    else:self.fix_dict.pop(self.golist_[index].name)
  
  def AddTrajectory(self,traj,go):
    self.trajlist_.append(traj)
    self.golist_.append(go)
    self.ehlist_.append(go.GetView().handle)
    self.index_dict[go.name]=len(self.golist_)-1
  
  def RemoveTrajectory(self,index):
    if type(index)==type(''):
      index=self.index_dict[index]
    self.trajlist_.pop(index)
    self.golist_.pop(index)
    self.ehlist_.pop(index)
  
  def SetSpeed(self,val):
  #Value should be between 0 and 1
    if not (val<=1. and val >=0.):
      print('Speed should be set between 0 and 1')
      return
    else:
      val=self._speed_slider_min-val*(self._speed_slider_min-self._speed_slider_max)
      self._SetSpeedSliderPos(val)
      print(val)
      



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
from ost import gui
from ost.gui import FileLoader

from PyQt5 import QtCore, QtGui, QtWidgets

class InitMenuBar(QtCore.QObject):
  def __init__(self, menu_bar=None):
    QtCore.QObject.__init__(self, menu_bar)
    self.scene_selection_ = gui.SceneSelection.Instance()
    
    test_action = QtWidgets.QAction('Test Menu Point', self)
    test_action.setStatusTip('Print Hello World')
    test_action.setShortcut('Ctrl+T')
    test_action.triggered.connect(self.TestMethod)

    test = menu_bar.addMenu('&Test')
    test.addAction(test_action)

  def TestMethod(self):
    reply = QtWidgets.QMessageBox()
    
    node_count = self.scene_selection_.GetActiveNodeCount()
    if(node_count > 0):
      string = "";
      for i in range(node_count):
        entity = self.scene_selection_.GetActiveNode(i)
        string=string + entity.GetName()
      reply.setText("Oh, there are selected entities: %s" % string)
    else:
      reply.setText("This is a test!")
    reply.addButton(QtWidgets.QMessageBox.Yes)
    reply.exec_()
    
menu_bar=gui.GostyApp.Instance().perspective.GetMenuBar()
InitMenuBar(menu_bar)



from PyQt5 import QtCore, QtWidgets
import threading
from ost import LogError
from ost import gui, gfx
from ost.io.remote import RemoteLoad, REMOTE_REPOSITORIES
import re

class RemoteLoader(QtWidgets.QWidget):
  def __init__(self):
    QtWidgets.QWidget.__init__(self)
    self._line = QtWidgets.QLineEdit(self)
    self._load = QtWidgets.QToolButton(self)
    self._line.returnPressed.connect(self._DoLoad)
    self._load.clicked.connect(self._DoLoad)
    hbox = QtWidgets.QHBoxLayout(self)
    hbox.addWidget(self._line,1)
    hbox.addWidget(self._load,0)
    self.setLayout(hbox)
    hbox.setContentsMargins(3,3,3,3)
    hbox.setSpacing(3)
    self._menu = self._RemoteMenu()
    self._load.setMenu(self._menu)
    self._current_repo = 'pdb'
    self._UpdateLoadButton(self._current_repo)
    self.setFixedHeight(self._load.height())
    self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)

  def _RemoteMenu(self):
    menu = QtWidgets.QMenu()
    action_group = QtWidgets.QActionGroup(menu)
    for k,v in REMOTE_REPOSITORIES.items():
      action = menu.addAction(v.name)
      action.setCheckable(True)
      if k == 'pdb':
        action.setChecked(True)
      action.setData(QtCore.QVariant(k))
      action_group.addAction(action)
    menu.triggered.connect(self._ToggleRepo)
    return menu
  def _UpdateLoadButton(self, current):
    name = REMOTE_REPOSITORIES[current].name
    self._load.setText('load from %s  ' % name)

  def _ToggleRepo(self, action):
    self._current_repo = str(action.data())
    self._UpdateLoadButton(self._current_repo)


  def _DoLoad(self):
    split_ids = []
    for p in str(self._line.text()).split(','):
      for k in p.split():
        if len(k.strip()):
          split_ids.append(k.strip())
    self._line.setText('')
    for split_id in split_ids:
      try:
        ent = RemoteLoad(split_id, from_repo=self._current_repo)
      except Exception as e:
        LogError(str(e))
        continue
      g = gfx.Entity(split_id, ent)
      try:
        gfx.Scene().Add(g)
      except Exception as e:
        LogError(str(e))

remote_loader=RemoteLoader()
remote_loader_for_panel=gui.Widget(remote_loader)
remote_loader_for_panel.qobject.setSizePolicy(QtWidgets.QSizePolicy.Expanding, 
                                              QtWidgets.QSizePolicy.Fixed)
panels=gui.GostyApp.Instance().perspective.panels
panels.AddWidgetToPool("Remote Loader",remote_loader_for_panel)



e=io.LoadPDB("../demos/data/sh2.pdb")
s=io.LoadSurface("../demos/data/sh2","msms")

scene.Add(gfx.Entity("x",gfx.HSC,e.Select("chain=A")))
scene["x"].SetColor(gfx.Color(0.9,0.95,1.0))
scene["x"].SetColor(gfx.Color(1.0,0.9,0.7),"rtype=helix")
scene["x"].SetColor(gfx.Color(0.5,1.0,0.7),"rtype=strand")

scene.Add(gfx.Surface("s",s))
scene["s"].SetOpacity(0.4)
scene["s"].SetMat(0,1,0,0)

scene.bg=gfx.WHITE



import unittest
from ost import *
from ost import settings
from ost import seq
from ost.bindings.clustalw import *

class TestGlobalAlign(unittest.TestCase):
  def testDeletionInSeqB(self):
    seq_a=seq.CreateSequence('A', 'aacdefghiklmn')
    seq_b=seq.CreateSequence('B', 'acdhiklmn')
    alns=seq.alg.GlobalAlign(seq_a, seq_b, seq.alg.BLOSUM62)
    self.assertEqual(len(alns), 1)
    self.assertEqual(alns[0].sequences[0].name, 'A')
    self.assertEqual(alns[0].sequences[1].name, 'B')
    self.assertEqual(str(alns[0].sequences[0]), 'aacdefghiklmn')
    self.assertEqual(str(alns[0].sequences[1]), '-acd---hiklmn')
    self.assertEqual(alns[0].sequences[0].offset, 0)
    self.assertEqual(alns[0].sequences[1].offset, 0)

  def testDeletionInSeqA(self):
    seq_a=seq.CreateSequence('A', 'acdhiklmn')
    seq_b=seq.CreateSequence('B', 'acdefghiklmn')
    alns=seq.alg.GlobalAlign(seq_a, seq_b, seq.alg.BLOSUM62)
    self.assertEqual(len(alns), 1)
    self.assertEqual(alns[0].sequences[0].name, 'A')
    self.assertEqual(alns[0].sequences[1].name, 'B')

    self.assertEqual(str(alns[0].sequences[0]), 'acd---hiklmn')
    self.assertEqual(str(alns[0].sequences[1]), 'acdefghiklmn')
    self.assertEqual(alns[0].sequences[0].offset, 0)
    self.assertEqual(alns[0].sequences[1].offset, 0)

  def testOffset(self):
    seq_a=seq.CreateSequence('A', 'acdhiklmn')
    seq_b=seq.CreateSequence('B', 'ggiklmn')
    alns=seq.alg.GlobalAlign(seq_a, seq_b, seq.alg.BLOSUM62)
    self.assertEqual(len(alns), 1)
    self.assertEqual(alns[0].sequences[0].name, 'A')
    self.assertEqual(alns[0].sequences[1].name, 'B')

    self.assertEqual(str(alns[0].sequences[0]), 'acdhiklmn')
    self.assertEqual(str(alns[0].sequences[1]), 'g--giklmn')
    self.assertEqual(alns[0].sequences[0].offset, 0)
    self.assertEqual(alns[0].sequences[1].offset, 0)

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
#
# Authors: Stefan Bienert 
#
from PyQt5 import QtCore, QtGui, QtWidgets
from ost.mol.alg import Superpose
from ost import mol

class ChainComboBox(QtWidgets.QComboBox):
  def __init__(self, ent, gfx, parent=None):
    # class variables
    self.all_chains = 'All'
    QtWidgets.QComboBox.__init__(self, parent)
    self.entity = ent
    self.addItem(self.all_chains)
    for chain in self.entity.chains:
      self.addItem(chain.name)
    if self.count()>0:
      self.setCurrentIndex(0)
    if gfx:
      self.gfx = gfx
      self.highlighted.connect(self._HighlightChain)
    else:
      self.gfx = None

  def focusOutEvent (self, event):
    if self.gfx:
      self.gfx.selection = None

  def SetItems(self, ent, gfx):
    self.clear()
    self.entity = ent
    self.addItem(self.all_chains)
    for chain in self.entity.chains:
      self.addItem(chain.name)
    if self.count()>0:
      self.setCurrentIndex(0)
    if gfx:
      self.gfx = gfx

  def _HighlightChain(self, chain_idx):
    chain = self.itemText(chain_idx)
    if chain != 'All':
      self.gfx.SetSelection(self.entity.Select('cname="%s"' % str(chain)))
    else:
      self.gfx.SetSelection(self.entity.Select(''))

  def _GetSelectedChain(self):
    if self.currentIndex() == -1:
      return mol.EntityHandle()
    elif self.currentText() == self.all_chains:
      return self.entity
    return self.entity.Select('cname="%s"' % str(self.currentText()))

  def _SetSelectedChain(self, chain):
    if hasattr(chain, 'name'):
      name = chain.name
    else:
      name = str(chain)
    for i in range(self.count()):
      if self.itemText(i) == name:
        self.setCurrentIndex(i)
        break
  selected_chain = property(_GetSelectedChain, _SetSelectedChain)

class SuperpositionDialog(QtWidgets.QDialog):
  """
  Provides a graphical user interface to structurally superpose two entities.
  Uses function :func:`~ost.mol.alg.Superpose`. The RMSD of two superposed
  molecules will be stored in attribute ``rmsd``. An index for the selected
  reference molecule will be stored in attribute ``reference``.

  :param ent_one: The first entity
  :type ent_one: :class:`~ost.mol.EntityView`, :class:`~ost.mol.EntityHandle`
                 or :class:`~ost.gfx.Entity`
  :param ent_two: The second entity
  :type ent_two: :class:`~ost.mol.EntityView`, :class:`~ost.mol.EntityHandle`
                 or :class:`~ost.gfx.Entity`

  **Example Usage:**

  .. code-block:: python

    e1=io.LoadPDB('examples/code_fragments/entity/pdb1ake.ent')
    e2=io.LoadPDB('examples/code_fragments/entity/pdb4ake.ent')

    sd = ost.gui.dng.superpositiondialog.SuperpositionDialog(e1, e2)

    g1=gfx.Entity('G1', e1)
    g2=gfx.Entity('G2', e2)
    scene.Add(g1)
    scene.Add(g2)

    if sd.reference == 0:
      scene.CenterOn(g1)
    else:
      scene.CenterOn(g2)

    if sd.rmsd != None:
      LogScript('RMSD: %.3f'%sd.rmsd)
  """

  def __init__(self, ent_one, ent_two, parent=None):
    # class variables
    self.rmsd_superposed_atoms = None
    self.rmsd = None
    self.fraction_superposed = None
    self.superposition_error = None
    self._mmethod_dict = {'number': 'number',
                          'index': 'index',
                          'local alignment': 'local-aln',
                          'global alignment': 'global-aln'}
    self.gfx_one = None
    self.gfx_two = None
    self.gfx_select_one = None
    self.gfx_select_two = None
    QtWidgets.QDialog.__init__(self, parent)
    self.setWindowTitle('Superpose Structures')
    if not isinstance(ent_one, mol.EntityHandle) and \
       not isinstance(ent_one, mol.EntityView):
      n_one = ent_one.GetName()
      self.gfx_one = ent_one
      self.gfx_select_one = self.gfx_one.GetSelection()
      self.ent_one = ent_one.GetView()
    else:
      if isinstance(ent_one, mol.EntityHandle):
        n_one = ent_one.GetName()
      elif isinstance(ent_one, mol.EntityView):
        n_one = ent_one.GetHandle().GetName()
      self.ent_one = ent_one
    if len(n_one) == 0:
      n_one = '1'
    if not isinstance(ent_two, mol.EntityHandle) and \
       not isinstance(ent_two, mol.EntityView):
      n_two = ent_two.GetName()
      self.gfx_two = ent_two
      self.gfx_select_two = self.gfx_two.GetSelection()
      self.ent_two = ent_two.GetView()
    else:
      if isinstance(ent_two, mol.EntityHandle):
        n_two = ent_two.GetName()
      elif isinstance(ent_two, mol.EntityView):
        n_two = ent_two.GetHandle().GetName()
      self.ent_two = ent_two
    if len(n_two) == 0:
      n_two = '2'
    if n_one == n_two:
      n_one = n_one + ' 1'
      n_two = n_two + ' 2' 
    layout = QtWidgets.QGridLayout(self)
    # select reference molecule
    self.reference = 0;
    self._reference = self._ReferenceSelection(n_one, n_two)
    grow = 0
    layout.addWidget(QtWidgets.QLabel("reference"), grow, 0)
    layout.addWidget(self._reference, grow, 1)
    grow += 1
    # chains
    self._chain_one = ChainComboBox(self.ent_one, self.gfx_one, self)
    self._chain_two = ChainComboBox(self.ent_two, self.gfx_two, self)
    layout.addWidget(QtWidgets.QLabel("reference chain"), grow, 0)
    layout.addWidget(self._chain_one, grow, 1)
    grow += 1
    layout.addWidget(QtWidgets.QLabel("chain"), grow, 0)
    layout.addWidget(self._chain_two, grow, 1)
    grow += 1
    # link chain and reference selection
    self._reference.currentIndexChanged.connect(self._ChangeChainSelection)
    # match methods
    self._methods = self._MatchMethods()
    layout.addWidget(QtWidgets.QLabel('match residues by'), grow, 0)
    grow += 1
    layout.addWidget(self._methods)

    # iterative
    self._iterative=None
    self._it_box, self._it_in, self._dist_in = self._ItBox()
    layout.addWidget(self._it_box, grow, 0)
    # atoms
    self._atoms = self._FetchAtoms(self._methods.size(),
                                   self.ent_one,
                                   self.ent_two)
    self._atmselectbx, self._atmselectgrp = self._AtomSelectionBox()
    layout.addWidget(self._atmselectbx, grow, 1)
    grow += 1
    # buttons
    ok_button = QtWidgets.QPushButton("Superpose")
    ok_button.clicked.connect(self.accept)
    cancel_button = QtWidgets.QPushButton("Cancel")
    hbox_layout = QtWidgets.QHBoxLayout()
    hbox_layout.addStretch(1)
    layout.addLayout(hbox_layout, grow, 0, 1, 2)
    grow += 1
    cancel_button.clicked.connect(self.reject)
    self.accepted.connect(self._Superpose)
    hbox_layout.addWidget(cancel_button, 0)
    hbox_layout.addWidget(ok_button, 0)
    ok_button.setDefault(True)
    self.exec_()
    # restore old selections
    if self.gfx_one:
      self.gfx_one.SetSelection(self.gfx_select_one)
    if self.gfx_two:
      self.gfx_two.SetSelection(self.gfx_select_two)

  def _Superpose(self):
    view_one = self._chain_one.selected_chain
    view_two = self._chain_two.selected_chain
    atoms = self._GetAtomSelection()
    try:
      sp = Superpose(view_two, view_one,
                     self._mmethod_dict[str(self._methods.currentText())],
                     atoms, iterative=self._iterative, 
                     max_iterations=self._it_in.value(), 
                     distance_threshold=self._dist_in.value())
    except Exception as e:
      # mark as failed by setting superposition_error and let caller handle it
      self.superposition_error = str(e)
      return
    self.rmsd = sp.rmsd
    if self._iterative:
      self.rmsd_superposed_atoms = sp.rmsd_superposed_atoms
      self.fraction_superposed = sp.fraction_superposed

  def _toggle_atoms(self, checked):
    if checked:
      self._atoms.setEnabled(True)
    else:
      self._atoms.setEnabled(False)

  def _AtomSelectionBox(self):
    bt1 = QtWidgets.QRadioButton('All')
    bt2 = QtWidgets.QRadioButton('Backbone')
    bt3 = QtWidgets.QRadioButton('CA')
    self.cstmbtntxt = 'Custom'
    custom_rbutton = QtWidgets.QRadioButton(self.cstmbtntxt)
    group = QtWidgets.QButtonGroup()
    group.addButton(bt1)
    group.addButton(bt2)
    group.addButton(bt3)
    group.addButton(custom_rbutton)
    bt1.setChecked(True)
    vbox_layout = QtWidgets.QVBoxLayout()
    vbox_layout.addWidget(bt1)
    vbox_layout.addWidget(bt2)
    vbox_layout.addWidget(bt3)
    vbox_layout.addWidget(custom_rbutton)
    vbox_layout.addWidget(self._atoms)
    custom_rbutton.toggled.connect(self._toggle_atoms)
    box = QtWidgets.QGroupBox("atom selection")
    box.setLayout(vbox_layout)
    return box, group

  def _GetAtomSelection(self):
    checkedbtn = self._atmselectgrp.checkedButton()
    if str(checkedbtn.text()) != self.cstmbtntxt:
      return str(checkedbtn.text())
    slctn_model = self._atoms.selectionModel()
    dt_model = slctn_model.model()
    atms = list()
    for idx in slctn_model.selectedRows():
      slctn = dt_model.data(idx, Qt.DisplayRole).toString()
      atms.append(str(slctn))
    return atms

  def _FetchAtoms(self, dim, ent_a, ent_b):
    # fetch list of atoms: only those which are in both entities are considered
    atm_dict = {}
    for atm in ent_a.GetAtomList():
      atm_dict[atm.name] = 0
    for atm in ent_b.GetAtomList():
      if atm.name in atm_dict:
        atm_dict[atm.name] = 1
    atmlst = list()
    for atm in sorted(atm_dict.keys()):
      if atm_dict[atm]:
        atmlst.append(atm)
    elems = QtCore.QStringListModel(atmlst)
    atoms = QtWidgets.QListView(self)
    dim.setHeight(3*dim.height())
    atoms.setFixedSize(dim)
    atoms.setModel(elems)
    atoms.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
    atoms.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
    atoms.setEnabled(False)
    return atoms

  def _ReferenceSelection(self, name_a, name_b):
    cbox = QtWidgets.QComboBox()
    cbox.addItem(name_a)
    cbox.addItem(name_b)
    if cbox.count() > 0:
      cbox.setCurrentIndex(0)
    return cbox

  def _toggle_iterative(self, checked):
    if checked:
      self._it_in.setEnabled(True)
      self._dist_in.setEnabled(True)
      self._iterative=True
    else:
      self._it_in.setEnabled(False)
      self._dist_in.setEnabled(False)
      self._iterative=False

  def _ItBox(self):
    bt1 = QtWidgets.QRadioButton("On")
    iteration_label=QtWidgets.QLabel("Max Iterations: ")
    distance_label=QtWidgets.QLabel("Dist Thresh: ")
    iteration_in=QtWidgets.QSpinBox()
    iteration_in.setRange(1,30)
    iteration_in.setValue(8)
    distance_in=QtWidgets.QDoubleSpinBox()
    distance_in.setRange(1.0,10.0)
    distance_in.setValue(3.0)
    distance_in.setDecimals(1)
    distance_in.setSingleStep(0.5)
    iteration_in.setEnabled(False)
    distance_in.setEnabled(False)
    bt1.setChecked(False)
    self._iterative=False
    vbox_layout = QtWidgets.QVBoxLayout()
    vbox_layout.addWidget(bt1)
    vbox_layout.addWidget(iteration_label)
    vbox_layout.addWidget(iteration_in)
    vbox_layout.addWidget(distance_label)
    vbox_layout.addWidget(distance_in)
    vbox_layout.addSpacing(50)
    bt1.toggled.connect(self._toggle_iterative)
    box = QtWidgets.QGroupBox("Iterative")
    box.setLayout(vbox_layout)
    return box,iteration_in, distance_in

  def _ChangeChainSelection(self, index):
    if index == 0:
      self._chain_one.SetItems(self.ent_one, self.gfx_one)
      self._chain_two.SetItems(self.ent_two, self.gfx_two)
      self.reference = 0;
    elif index == 1:
      self._chain_one.SetItems(self.ent_two, self.gfx_two)
      self._chain_two.SetItems(self.ent_one, self.gfx_one)
      self.reference = 1;

  def _MatchMethods(self):
    methods=QtWidgets.QComboBox(self)
    for method in sorted(self._mmethod_dict):
      methods.addItem(method)
    return methods
